"""Auxiliary constants that are used in the library."""

__author__ = "Ederson Corbari"
__email__ = "e@neural7.io"
__copyright__ = "Copyright Neural7 2022, Big 5 Personality Traits"
__credits__ = ["John A. Johnson", "Dhiru Kholia"]
__license__ = "MIT"
__version__ = "1.0.0"
__status__ = "production"

from enum import Enum, IntEnum


class QuestionNumber(IntEnum):
    """Questionnaire types."""

    IPIP_300 = 300
    IPIP_120 = 120


class FacetScale(IntEnum):
    """Facet scale."""

    IPIP_MAX = 30
    IPIP_300 = 10
    IPIP_120 = 4


class FacetLevel(IntEnum):
    """Score level of a facet."""

    HIGH = 55
    LOW = 45


class NormScale(IntEnum):
    """Norms of the maximum and minimum facet."""

    CONST_MAX = 73
    CONST_MIN = 32


class NormCubic(float, Enum):
    """Perceptile cubic approximation of facets."""

    CONST1 = 210.335958661391
    CONST2 = 16.7379362643389
    CONST3 = 0.405936512733332
    CONST4 = 0.00270624341822222


class Big5Neuroticism(str, Enum):
    """Composition of the big5 facets: Neuroticism."""

    TRAIT1 = "Anxiety"
    TRAIT2 = "Anger"
    TRAIT3 = "Depression"
    TRAIT4 = "Self-Consciousness"
    TRAIT5 = "Immoderation"
    TRAIT6 = "Vulnerability"


class Big5Extraversion(str, Enum):
    """Composition of the big5 facets: Extraversion."""

    TRAIT1 = "Friendliness"
    TRAIT2 = "Gregariousness"
    TRAIT3 = "Assertiveness"
    TRAIT4 = "Activity-Level"
    TRAIT5 = "Excitement-Seeking"
    TRAIT6 = "Cheerfulness"


class Big5Openness(str, Enum):
    """Composition of the big5 facets: Openness."""

    TRAIT1 = "Imagination"
    TRAIT2 = "Artistic-Interests"
    TRAIT3 = "Emotionality"
    TRAIT4 = "Adventurousness"
    TRAIT5 = "Intellect"
    TRAIT6 = "Liberalism"


class Big5Agreeableness(str, Enum):
    """Composition of the big5 facets: Agreeableness."""

    TRAIT1 = "Trust"
    TRAIT2 = "Morality"
    TRAIT3 = "Altruism"
    TRAIT4 = "Cooperation"
    TRAIT5 = "Modesty"
    TRAIT6 = "Sympathy"


class Big5Conscientiousness(str, Enum):
    """Composition of the big5 facets: Conscientiousness."""

    TRAIT1 = "Self-Efficacy"
    TRAIT2 = "Orderliness"
    TRAIT3 = "Dutifulness"
    TRAIT4 = "Achievement-Striving"
    TRAIT5 = "Self-Discipline"
    TRAIT6 = "Cautiousness"



"""Unit tests for Utility."""

import json
import unittest

from ipipneo.utility import (answers_is_valid, big5_ocean_is_valid,
                             big5_target, create_big5_dict, organize_list_json,
                             raise_if_age_is_invalid, raise_if_sex_is_invalid,
                             reverse_scored)


def load_mock_answers(idx: int) -> dict:
    if idx == 1:
        name = "answers-test-1.json"
    elif idx == 2:
        name = "answers-test-2.json"
    with open(f"test/mock/{name}") as f:
        data = json.load(f)
    return data


class TestUtility(unittest.TestCase):

    maxDiff = None

    def test_raise_if_sex_is_invalid(self) -> None:
        with self.assertRaises(BaseException):
            raise_if_sex_is_invalid(sex="")

        with self.assertRaises(AssertionError):
            raise_if_sex_is_invalid(sex="m")

        with self.assertRaises(AssertionError):
            raise_if_sex_is_invalid(sex="f")

        with self.assertRaises(AssertionError):
            raise_if_sex_is_invalid(sex="male")

        with self.assertRaises(AssertionError):
            raise_if_sex_is_invalid(sex="female")

        self.assertEqual(raise_if_sex_is_invalid(sex="M"), True)
        self.assertEqual(raise_if_sex_is_invalid(sex="F"), True)

    def test_raise_if_age_is_invalid(self) -> None:
        with self.assertRaises(BaseException):
            raise_if_age_is_invalid(age="")

        with self.assertRaises(AssertionError):
            raise_if_age_is_invalid(age="40")

        with self.assertRaises(AssertionError):
            raise_if_age_is_invalid(age=-1)

        with self.assertRaises(AssertionError):
            raise_if_age_is_invalid(age=5)

        with self.assertRaises(AssertionError):
            raise_if_age_is_invalid(age=121)

        with self.assertRaises(AssertionError):
            raise_if_age_is_invalid(age=150)

        with self.assertRaises(AssertionError):
            raise_if_age_is_invalid(age=300)

        self.assertEqual(raise_if_age_is_invalid(age=18), True)
        self.assertEqual(raise_if_age_is_invalid(age=21), True)
        self.assertEqual(raise_if_age_is_invalid(age=25), True)
        self.assertEqual(raise_if_age_is_invalid(age=30), True)
        self.assertEqual(raise_if_age_is_invalid(age=35), True)
        self.assertEqual(raise_if_age_is_invalid(age=40), True)
        self.assertEqual(raise_if_age_is_invalid(age=49), True)
        self.assertEqual(raise_if_age_is_invalid(age=50), True)
        self.assertEqual(raise_if_age_is_invalid(age=61), True)
        self.assertEqual(raise_if_age_is_invalid(age=80), True)
        self.assertEqual(raise_if_age_is_invalid(age=90), True)
        self.assertEqual(raise_if_age_is_invalid(age=100), True)

    def test_answers_is_valid(self) -> None:
        with self.assertRaises(BaseException):
            answers_is_valid(answers="")

        with self.assertRaises(BaseException):
            answers_is_valid(answers=[])

        with self.assertRaises(BaseException):
            answers_is_valid(answers=["A"])

        with self.assertRaises(BaseException):
            answers_is_valid(answers=[1, 2, 3, 4, 5])

        with self.assertRaises(BaseException):
            answers_is_valid(answers=[-1, -2, -3, -4, -5])

        with self.assertRaises(BaseException):
            answers_is_valid(answers=[0, 5, 10])

        with self.assertRaises(BaseException):
            answers_is_valid(answers=list(range(1, 121)))

        self.assertEqual(
            answers_is_valid(
                answers=[
                    1,
                    2,
                    3,
                    4,
                    3,
                    2,
                    2,
                    3,
                    3,
                    2,
                    3,
                    3,
                    3,
                    2,
                    3,
                    2,
                    1,
                    1,
                    4,
                    3,
                    4,
                    4,
                    5,
                    1,
                    4,
                    2,
                    3,
                    3,
                    3,
                    2,
                    4,
                    4,
                    1,
                    3,
                    3,
                    2,
                    4,
                    1,
                    3,
                    3,
                    5,
                    5,
                    4,
                    4,
                    3,
                    2,
                    3,
                    4,
                    2,
                    1,
                    3,
                    4,
                    1,
                    2,
                    3,
                    5,
                    2,
                    3,
                    4,
                    3,
                    2,
                    3,
                    2,
                    3,
                    3,
                    2,
                    3,
                    3,
                    3,
                    4,
                    3,
                    1,
                    3,
                    3,
                    3,
                    2,
                    3,
                    5,
                    2,
                    1,
                    2,
                    3,
                    2,
                    1,
                    4,
                    1,
                    3,
                    2,
                    1,
                    3,
                    2,
                    3,
                    5,
                    2,
                    4,
                    1,
                    3,
                    4,
                    3,
                    3,
                    2,
                    4,
                    3,
                    4,
                    4,
                    3,
                    2,
                    1,
                    2,
                    3,
                    3,
                    4,
                    1,
                    4,
                    4,
                    3,
                    3,
                    3,
                    4,
                    3,
                ]
            ),
            True,
        )

        a = [
            5,
            2,
            3,
            2,
            5,
            4,
            1,
            3,
            5,
            5,
            3,
            2,
            2,
            4,
            5,
            2,
            5,
            4,
            2,
            5,
            2,
            4,
            5,
            2,
            4,
            1,
            2,
            5,
            4,
            4,
            2,
            3,
            4,
            5,
            5,
            4,
            2,
            2,
            5,
            5,
            2,
            2,
            4,
            3,
            4,
            4,
            4,
            4,
            5,
            4,
            2,
            4,
            3,
            3,
            4,
            5,
            2,
            4,
            1,
            3,
            1,
            3,
            1,
            4,
            4,
            2,
            2,
            3,
            5,
            5,
            2,
            4,
            4,
            4,
            3,
            1,
            2,
            4,
            5,
            5,
            3,
            2,
            2,
            2,
            5,
            1,
            4,
            4,
            2,
            1,
            4,
            2,
            2,
            4,
            3,
            2,
            1,
            3,
            5,
            5,
            1,
            4,
            2,
            2,
            4,
            4,
            3,
            4,
            4,
            5,
            3,
            2,
            4,
            5,
            5,
            1,
            4,
            2,
            4,
            4,
        ]

        b = organize_list_json(answers=load_mock_answers(idx=1))
        assert isinstance(b, list), "b must be a list"
        self.assertEqual(a, b)

        c = organize_list_json(answers=load_mock_answers(idx=2))
        assert isinstance(c, list), "c must be a list"
        self.assertEqual(c, b)

    def test_organize_list_json(self) -> None:
        with self.assertRaises(AssertionError):
            organize_list_json(answers="")

        with self.assertRaises(BaseException):
            organize_list_json(answers={"A": []})

        with self.assertRaises(BaseException):
            organize_list_json(answers={"answers": []})

        with self.assertRaises(BaseException):
            organize_list_json(answers={"answers": [{"id_question": 1}]})

        with self.assertRaises(BaseException):
            organize_list_json(answers={"answers": [{"id_select": 1}]})

        data = {
            "answers": [
                {"id_question": 1, "id_select": 1},
                {"id_question": 2, "id_select": 2},
                {"id_question": 3, "id_select": 3},
                {"id_question": 4, "id_select": 4},
                {"id_question": 5, "id_select": 5},
            ]
        }

        answers = organize_list_json(answers=data)
        assert isinstance(answers, list), "answers must be a list"
        self.assertEqual(len(answers), 5)

    def test_big5_ocean_is_valid(self) -> None:
        with self.assertRaises(BaseException) as e:
            big5_ocean_is_valid(label="")
        self.assertEqual(str(e.exception), "The Big-Five label is invalid!")

        with self.assertRaises(BaseException) as e:
            big5_ocean_is_valid(label="OCEAN")
        self.assertEqual(str(e.exception), "The Big-Five label is invalid!")

        self.assertEqual(big5_ocean_is_valid(label="O"), True)
        self.assertEqual(big5_ocean_is_valid(label="C"), True)
        self.assertEqual(big5_ocean_is_valid(label="E"), True)
        self.assertEqual(big5_ocean_is_valid(label="A"), True)
        self.assertEqual(big5_ocean_is_valid(label="N"), True)

    def test_big5_target(self) -> None:
        with self.assertRaises(BaseException) as e:
            big5_target(label="")
        self.assertEqual(str(e.exception), "The Big-Five label is invalid!")

        O = list(map(str, big5_target(label="O")))
        C = list(map(str, big5_target(label="C")))
        E = list(map(str, big5_target(label="E")))
        A = list(map(str, big5_target(label="A")))
        N = list(map(str, big5_target(label="N")))

        self.assertEqual(len(O), 6)
        self.assertEqual(len(C), 6)
        self.assertEqual(len(E), 6)
        self.assertEqual(len(A), 6)
        self.assertEqual(len(N), 6)

    def test_create_big5_dict(self) -> None:
        N = {
            "label": "N",
            "big5": 37.23631660596993,
            "x": [
                0,
                55.29153291804727,
                52.068589887460575,
                30.04288206625688,
                39.69850290396829,
                28.601999121511483,
                38.536490293892996,
            ],
            "y": [
                0,
                "average",
                "average",
                "low",
                "average",
                "low",
                "average",
            ],
        }
        N = create_big5_dict(
            label=N.get("label"), big5=N.get("big5"), x=N.get("x"), y=N.get("y")
        )
        assert isinstance(N, dict), "N must be a dict"
        self.assertEqual(N.get("N"), 37.23631660596993)
        assert isinstance(N.get("traits"), list), "traits must be a list"

        E = {
            "label": "E",
            "big5": 21.46713047820697,
            "x": [
                0,
                20.788746198012063,
                8.857170154068399,
                25.365062122504668,
                70.68785093930268,
                36.25782829248527,
                27.560328329400278,
            ],
            "y": [
                0,
                "low",
                "low",
                "low",
                "high",
                "average",
                "low",
            ],
        }

        E = create_big5_dict(
            label=E.get("label"), big5=E.get("big5"), x=E.get("x"), y=E.get("y")
        )
        assert isinstance(E, dict), "N must be a dict"
        self.assertEqual(E.get("E"), 21.46713047820697)
        assert isinstance(E.get("traits"), list), "traits must be a list"

        O = {
            "label": "O",
            "big5": 26.882298529738506,
            "x": [
                0,
                1,
                14.832308817371768,
                22.458638690321635,
                78.39980163147527,
                27.6351617847169,
                79.99472618213304,
            ],
            "y": [
                0,
                "low",
                "low",
                "low",
                "high",
                "low",
                "high",
            ],
        }

        O = create_big5_dict(
            label=O.get("label"), big5=O.get("big5"), x=O.get("x"), y=O.get("y")
        )
        assert isinstance(O, dict), "O must be a dict"
        self.assertEqual(O.get("O"), 26.882298529738506)
        assert isinstance(O.get("traits"), list), "traits must be a list"

        A = {
            "label": "A",
            "big5": 57.52658875205816,
            "x": [
                0,
                70.5561195632738,
                89.87557658724631,
                22.561476501041227,
                63.50546308998889,
                55.67124528467815,
                21.651269458084244,
            ],
            "y": [
                0,
                "high",
                "high",
                "low",
                "average",
                "average",
                "low",
            ],
        }

        A = create_big5_dict(
            label=A.get("label"), big5=A.get("big5"), x=A.get("x"), y=A.get("y")
        )
        assert isinstance(A, dict), "A must be a dict"
        self.assertEqual(A.get("A"), 57.52658875205816)
        assert isinstance(A.get("traits"), list), "traits must be a list"

        C = {
            "label": "C",
            "big5": 86.96687032595662,
            "x": [
                0,
                67.18081968991703,
                98.82696217870512,
                54.122030713014226,
                84.86323430898972,
                92.95956099997227,
                38.524355262005315,
            ],
            "y": [
                0,
                "average",
                "high",
                "average",
                "high",
                "high",
                "average",
            ],
        }

        C = create_big5_dict(
            label=C.get("label"), big5=C.get("big5"), x=C.get("x"), y=C.get("y")
        )
        assert isinstance(C, dict), "A must be a dict"
        self.assertEqual(C.get("C"), 86.96687032595662)
        assert isinstance(C.get("traits"), list), "traits must be a list"

    def test_reverse_scored(self) -> None:
        with self.assertRaises(TypeError) as e:
            reverse_scored(a=1)

        with self.assertRaises(BaseException) as e:
            reverse_scored(select=900)
        self.assertEqual(
            str(e.exception), "Something wrong in the selection option: 900"
        )

        self.assertEqual(reverse_scored(select=1), 5)
        self.assertEqual(reverse_scored(select=2), 4)
        self.assertEqual(reverse_scored(select=3), 3)
        self.assertEqual(reverse_scored(select=4), 2)
        self.assertEqual(reverse_scored(select=5), 1)



"""Creation and scoring of the facets that make up the Big-Five."""

__author__ = "Ederson Corbari"
__email__ = "e@neural7.io"
__copyright__ = "Copyright Neural7 2022, Big 5 Personality Traits"
__credits__ = ["John A. Johnson", "Dhiru Kholia"]
__license__ = "MIT"
__version__ = "1.0.0"
__status__ = "production"

from enum import IntEnum

from ipipneo.model import (FacetLevel, FacetScale, NormCubic, NormScale,
                           QuestionNumber)
from ipipneo.utility import big5_ocean_is_valid, create_big5_dict


class Facet:
    """Creating and scoring facets and Big-Five."""

    def __init__(self, nquestion: IntEnum) -> None or BaseException:
        """
        Initialize the class.

        Args:
            - nquestion: Enum with question type.
        """
        if nquestion == 300:
            self._scale = FacetScale.IPIP_300.value
        elif nquestion == 120:
            self._scale = FacetScale.IPIP_120.value
        else:
            raise BaseException(f"The available questions are: {list(QuestionNumber)}")

    def score(self, answers: list) -> list or BaseException:
        """
        Score facet scales are created.

        Args:
            - answers: The list with the answers.
        """
        answers.insert(0, 0)
        ss = [0] * len(answers)

        try:
            for j in range(FacetScale.IPIP_MAX.value):
                for i in range(self._scale):
                    ss[1 + j] += answers[1 + i * FacetScale.IPIP_MAX.value + j]
        except IndexError as e:
            raise BaseException(f"The number of questions setting is wrong: {str(e)}")

        return ss

    def b5create(self, score: list) -> dict:
        """
        Numbers each facet set from 1 to 6 to create the Big-Five (OCEAN).

        Args:
            - score: The facet score result.
        """
        ss, j = score, 0

        N = [0] * len(ss)
        E = [0] * len(ss)
        O = [0] * len(ss)
        A = [0] * len(ss)
        C = [0] * len(ss)

        try:
            for i in range(1, 7):
                N[i] = ss[i + j]
                E[i] = ss[i + j + 1]
                O[i] = ss[i + j + 2]
                A[i] = ss[i + j + 3]
                C[i] = ss[i + j + 4]
                j = j + 4
        except IndexError as e:
            raise BaseException(f"The number of questions setting is wrong: {str(e)}")

        return {"O": O, "C": C, "E": E, "A": A, "N": N}

    def domain(self, score: list) -> dict:
        """
        Create the Big-Five (OCEAN) domain score.

        Args:
            - score: The facet score result.
        """
        try:
            N = score[1] + score[6] + score[11] + score[16] + score[21] + score[26]
            E = score[2] + score[7] + score[12] + score[17] + score[22] + score[27]
            O = score[3] + score[8] + score[13] + score[18] + score[23] + score[28]
            A = score[4] + score[9] + score[14] + score[19] + score[24] + score[29]
            C = score[5] + score[10] + score[15] + score[20] + score[25] + score[30]
        except IndexError as e:
            raise BaseException(f"Invalid position in the score array: {str(e)}")

        return {"O": O, "C": C, "E": E, "A": A, "N": N}

    def distrib(self, size: int, b5: dict, norm: dict) -> list:
        """
        Creation and distribution with weights.

        Args:
            - size: Vector size, must be the same as response size.
            - b5: Dictionary with composition of the Big-Five.
            - norm: Dictionary with the calculation of norms.
        """
        N = [0] * size
        E = [0] * size
        O = [0] * size
        A = [0] * size
        C = [0] * size

        try:
            for i in range(1, 7):
                N[i] = 50 + (
                    10
                    * (b5.get("N")[i] - norm.get("ns")[i + 10])
                    / norm.get("ns")[i + 16]
                )
                E[i] = 50 + (
                    10
                    * (b5.get("E")[i] - norm.get("ns")[i + 22])
                    / norm.get("ns")[i + 28]
                )
                O[i] = 50 + (
                    10
                    * (b5.get("O")[i] - norm.get("ns")[i + 34])
                    / norm.get("ns")[i + 40]
                )
                A[i] = 50 + (
                    10
                    * (b5.get("A")[i] - norm.get("ns")[i + 46])
                    / norm.get("ns")[i + 52]
                )
                C[i] = 50 + (
                    10
                    * (b5.get("C")[i] - norm.get("ns")[i + 58])
                    / norm.get("ns")[i + 64]
                )
        except IndexError as e:
            raise BaseException(f"The number of questions setting is wrong: {str(e)}")

        return {"O": O, "C": C, "E": E, "A": A, "N": N}

    def personality(self, size: int, big5: dict, traits: dict, label: str) -> dict:
        """
        Calculate the personalities / facets for each Big-Five.

        Args:
            - size: Vector size, must be the same as response size.
            - big5: Dictionary of normalized Big-Five.
            - traits: Dictionary with the distribution of personalities.
            - label: Capital letter referring to the Big-Five to be calculated.
        """
        big5_ocean_is_valid(label=label)

        big5, traits = big5.get(label, 0), traits.get(label, [])

        X = [0] * size
        Y = [0] * size

        try:
            for i in range(1, 7):
                Y[i] = traits[i]

                if traits[i] < FacetLevel.LOW.value:
                    Y[i] = "low"
                if (
                    traits[i] >= FacetLevel.LOW.value
                    and traits[i] <= FacetLevel.HIGH.value
                ):
                    Y[i] = "average"
                if traits[i] > FacetLevel.HIGH.value:
                    Y[i] = "high"

                X[i] = (
                    NormCubic.CONST1.value
                    - (NormCubic.CONST2.value * traits[i])
                    + (NormCubic.CONST3.value * traits[i] ** 2)
                    - (NormCubic.CONST4.value * traits[i] ** 3)
                )

                if traits[i] < NormScale.CONST_MIN.value:
                    X[i] = 1

                if traits[i] > NormScale.CONST_MAX.value:
                    X[i] = 99
        except IndexError as e:
            raise BaseException(f"The number of questions setting is wrong: {str(e)}")

        return create_big5_dict(label=label, big5=big5, x=X, y=Y) or {}



"""Unit tests for IpipNeo."""

import json
import unittest

from ipipneo.ipipneo import IpipNeo


def load_mock_answers_120() -> dict:
    with open("test/mock/answers-test-1.json") as f:
        data = json.load(f)
    return data


def load_mock_answers_300() -> dict:
    with open("test/mock/answers-test-4.json") as f:
        data = json.load(f)
    return data


def load_mock_answers_custom() -> dict:
    with open(f"test/mock/answers-test-6.json") as f:
        data = json.load(f)
    return data


class TestIpipNeo(unittest.TestCase):

    maxDiff = None

    def test_invalid_params(self) -> None:
        with self.assertRaises(BaseException):
            IpipNeo(question="")

        with self.assertRaises(BaseException):
            IpipNeo(question="OCEAN")

        with self.assertRaises(BaseException):
            IpipNeo(question=0)

        self.assertEqual(IpipNeo(question=120).__class__.__name__, "IpipNeo")
        self.assertEqual(IpipNeo(question=300).__class__.__name__, "IpipNeo")
        self.assertEqual(IpipNeo(question=120, test=True).__class__.__name__, "IpipNeo")
        self.assertEqual(IpipNeo(question=300, test=True).__class__.__name__, "IpipNeo")

    def test_compute_120(self) -> None:
        big5 = IpipNeo(question=120)

        with self.assertRaises(TypeError):
            big5.compute(sex="")

        with self.assertRaises(TypeError):
            big5.compute("")

        big5 = None

        #############################################
        # 1. Test with 40 year old man.
        #############################################
        result = IpipNeo(question=120).compute(
            sex="M", age=40, answers=load_mock_answers_120()
        )

        self.assertTrue(len(result.get("id")))
        self.assertTrue(len(result.get("theory")))
        self.assertTrue(len(result.get("model")))

        self.assertEqual(result.get("question"), 120)
        self.assertEqual(result.get("person").get("sex"), "M")
        self.assertEqual(result.get("person").get("age"), 40)

        self.assertTrue(len(result.get("person").get("result")))
        self.assertTrue(len(result.get("person").get("result").get("personalities")))

        personalities = result.get("person").get("result").get("personalities")
        self.assertEqual(len(personalities), 5)

        self.assertEqual(personalities[0]["Openness"].get("O"), 13.262383714475419)
        self.assertEqual(
            personalities[1]["Conscientiousness"].get("C"), 18.62283542366208
        )
        self.assertEqual(personalities[2]["Extraversion"].get("E"), 34.10779114957646)
        self.assertEqual(personalities[3]["Agreeableness"].get("A"), 1.9889215690788262)
        self.assertEqual(personalities[4]["Neuroticism"].get("N"), 58.6632925696303)

        #############################################
        # 2. Test with 30 year old woman.
        #############################################
        result = IpipNeo(question=120).compute(
            sex="F", age=30, answers=load_mock_answers_120()
        )

        self.assertTrue(len(result.get("id")))
        self.assertTrue(len(result.get("theory")))
        self.assertTrue(len(result.get("model")))

        self.assertEqual(result.get("question"), 120)
        self.assertEqual(result.get("person").get("sex"), "F")
        self.assertEqual(result.get("person").get("age"), 30)

        self.assertTrue(len(result.get("person").get("result")))
        self.assertTrue(len(result.get("person").get("result").get("personalities")))

        personalities = result.get("person").get("result").get("personalities")
        self.assertEqual(len(personalities), 5)

        self.assertEqual(personalities[0]["Openness"].get("O"), 8.876064362586419)
        self.assertEqual(
            personalities[1]["Conscientiousness"].get("C"), 13.877369975064141
        )
        self.assertEqual(personalities[2]["Extraversion"].get("E"), 29.240137885780882)
        self.assertEqual(personalities[3]["Agreeableness"].get("A"), 1)
        self.assertEqual(personalities[4]["Neuroticism"].get("N"), 47.49034742252866)

        #############################################
        # 3. Test with 65 year old woman.
        #############################################
        result = IpipNeo(question=120).compute(
            sex="F", age=65, answers=load_mock_answers_120(), compare=True
        )

        self.assertIn("id", result)
        self.assertIn("theory", result)
        self.assertIn("model", result)
        self.assertIn("question", result)
        self.assertIn("person", result)

        self.assertIn("sex", result.get("person"))
        self.assertIn("age", result.get("person"))

        self.assertTrue(len(result.get("id")))
        self.assertTrue(len(result.get("theory")))
        self.assertTrue(len(result.get("model")))

        self.assertEqual(result.get("question"), 120)
        self.assertEqual(result.get("person").get("sex"), "F")
        self.assertEqual(result.get("person").get("age"), 65)

        self.assertTrue(len(result.get("person").get("result")))
        self.assertTrue(len(result.get("person").get("result").get("personalities")))
        self.assertTrue(len(result.get("person").get("result").get("compare")))

        personalities = result.get("person").get("result").get("personalities")
        self.assertEqual(len(personalities), 5)

        compare = result.get("person").get("result").get("compare")
        self.assertIn("user_answers_original", compare)
        self.assertIn("user_answers_reversed", compare)

        self.assertEqual(personalities[0]["Openness"].get("O"), 24.29091080263288)
        self.assertEqual(personalities[1]["Conscientiousness"].get("C"), 1)
        self.assertEqual(personalities[2]["Extraversion"].get("E"), 32.92660962191414)
        self.assertEqual(personalities[3]["Agreeableness"].get("A"), 1)
        self.assertEqual(personalities[4]["Neuroticism"].get("N"), 67.59105722337824)

        original = (
            result.get("person")
            .get("result")
            .get("compare")
            .get("user_answers_original")
        )
        assert isinstance(original, list), "original must be a list"
        self.assertEqual(len(original), 120)

        reversed = (
            result.get("person")
            .get("result")
            .get("compare")
            .get("user_answers_reversed")
        )
        assert isinstance(reversed, list), "reversed must be a list"
        self.assertEqual(len(reversed), 120)

        a = [x.get("id_select") for x in original]
        assert isinstance(original, list), "reversed must be a list"

        b = [x.get("id_select") for x in reversed]
        assert isinstance(reversed, list), "reversed must be a list"

        self.assertNotEqual(a, b)

        self.assertEqual(a[0], b[0])
        self.assertEqual(a[1], b[1])
        self.assertEqual(a[2], b[2])
        self.assertEqual(a[3], b[3])
        self.assertEqual(a[4], b[4])
        self.assertEqual(a[5], b[5])
        self.assertEqual(a[6], b[6])
        self.assertEqual(a[7], b[7])

        self.assertNotEqual(a[8], b[8])
        self.assertEqual(a[8], 5)
        self.assertEqual(b[8], 1)

        self.assertNotEqual(a[18], b[18])
        self.assertEqual(a[18], 2)
        self.assertEqual(b[18], 4)

        self.assertNotEqual(a[23], b[23])
        self.assertEqual(a[23], 2)
        self.assertEqual(b[23], 4)

        self.assertNotEqual(a[29], b[29])
        self.assertEqual(a[29], 4)
        self.assertEqual(b[29], 2)

        self.assertNotEqual(a[38], b[38])
        self.assertEqual(a[38], 5)
        self.assertEqual(b[38], 1)

        self.assertNotEqual(a[39], b[39])
        self.assertEqual(a[39], 5)
        self.assertEqual(b[39], 1)

        self.assertNotEqual(a[47], b[47])
        self.assertEqual(a[47], 4)
        self.assertEqual(b[47], 2)

        self.assertNotEqual(a[48], b[48])
        self.assertEqual(a[48], 5)
        self.assertEqual(b[48], 1)

        self.assertNotEqual(a[50], b[50])
        self.assertEqual(a[50], 2)
        self.assertEqual(b[50], 4)

        self.assertNotEqual(a[66], b[66])
        self.assertEqual(a[66], 2)
        self.assertEqual(b[66], 4)

    def test_compute_300(self) -> None:
        #############################################
        # 1. Test with 40 year old man.
        #############################################
        result = IpipNeo(question=300).compute(
            sex="M", age=40, answers=load_mock_answers_300()
        )

        self.assertTrue(len(result.get("id")))
        self.assertTrue(len(result.get("theory")))
        self.assertTrue(len(result.get("model")))

        self.assertEqual(result.get("question"), 300)
        self.assertEqual(result.get("person").get("sex"), "M")
        self.assertEqual(result.get("person").get("age"), 40)

        self.assertTrue(len(result.get("person").get("result")))
        self.assertTrue(len(result.get("person").get("result").get("personalities")))

        personalities = result.get("person").get("result").get("personalities")
        self.assertEqual(len(personalities), 5)

        self.assertEqual(personalities[0]["Openness"].get("O"), 38.846625568407774)
        self.assertEqual(
            personalities[1]["Conscientiousness"].get("C"), 4.6266733183044835
        )
        self.assertEqual(personalities[2]["Extraversion"].get("E"), 58.00148636052478)
        self.assertEqual(personalities[3]["Agreeableness"].get("A"), 11.934721812005705)
        self.assertEqual(personalities[4]["Neuroticism"].get("N"), 78.82240987792443)

        #############################################
        # 2. Test with 30 year old woman.
        #############################################
        result = IpipNeo(question=300).compute(
            sex="F", age=30, answers=load_mock_answers_300()
        )

        self.assertTrue(len(result.get("id")))
        self.assertTrue(len(result.get("theory")))
        self.assertTrue(len(result.get("model")))

        self.assertEqual(result.get("question"), 300)
        self.assertEqual(result.get("person").get("sex"), "F")
        self.assertEqual(result.get("person").get("age"), 30)

        self.assertTrue(len(result.get("person").get("result")))
        self.assertTrue(len(result.get("person").get("result").get("personalities")))

        personalities = result.get("person").get("result").get("personalities")
        self.assertEqual(len(personalities), 5)

        self.assertEqual(personalities[0]["Openness"].get("O"), 28.963387129747275)
        self.assertEqual(personalities[1]["Conscientiousness"].get("C"), 1)
        self.assertEqual(personalities[2]["Extraversion"].get("E"), 54.26151488253112)
        self.assertEqual(personalities[3]["Agreeableness"].get("A"), 1)
        self.assertEqual(personalities[4]["Neuroticism"].get("N"), 71.14096931653722)

        #############################################
        # 3. Test with 65 year old woman.
        #############################################
        result = IpipNeo(question=300).compute(
            sex="F", age=65, answers=load_mock_answers_300(), compare=True
        )

        self.assertIn("id", result)
        self.assertIn("theory", result)
        self.assertIn("model", result)
        self.assertIn("question", result)
        self.assertIn("person", result)

        self.assertIn("sex", result.get("person"))
        self.assertIn("age", result.get("person"))

        self.assertTrue(len(result.get("id")))
        self.assertTrue(len(result.get("theory")))
        self.assertTrue(len(result.get("model")))

        self.assertEqual(result.get("question"), 300)
        self.assertEqual(result.get("person").get("sex"), "F")
        self.assertEqual(result.get("person").get("age"), 65)

        self.assertTrue(len(result.get("person").get("result")))
        self.assertTrue(len(result.get("person").get("result").get("personalities")))
        self.assertTrue(len(result.get("person").get("result").get("compare")))

        personalities = result.get("person").get("result").get("personalities")
        self.assertEqual(len(personalities), 5)

        compare = result.get("person").get("result").get("compare")
        self.assertIn("user_answers_original", compare)
        self.assertIn("user_answers_reversed", compare)

        self.assertEqual(personalities[0]["Openness"].get("O"), 28.963387129747275)
        self.assertEqual(personalities[1]["Conscientiousness"].get("C"), 1)
        self.assertEqual(personalities[2]["Extraversion"].get("E"), 54.26151488253112)
        self.assertEqual(personalities[3]["Agreeableness"].get("A"), 1)
        self.assertEqual(personalities[4]["Neuroticism"].get("N"), 71.14096931653722)

        original = (
            result.get("person")
            .get("result")
            .get("compare")
            .get("user_answers_original")
        )
        assert isinstance(original, list), "original must be a list"
        self.assertEqual(len(original), 300)

        reversed = (
            result.get("person")
            .get("result")
            .get("compare")
            .get("user_answers_reversed")
        )
        assert isinstance(reversed, list), "reversed must be a list"
        self.assertEqual(len(reversed), 300)

        a = [x.get("id_select") for x in original]
        assert isinstance(original, list), "reversed must be a list"

        b = [x.get("id_select") for x in reversed]
        assert isinstance(reversed, list), "reversed must be a list"

        self.assertNotEqual(a, b)

    def test_compute_test_custom_reversed(self) -> None:
        #############################################
        # 1. Test with 40 year old man.
        #############################################
        result = IpipNeo(question=120, test=True).compute(
            sex="M", age=40, answers=load_mock_answers_custom(), compare=True
        )

        self.assertIn("id", result)
        self.assertIn("theory", result)
        self.assertIn("model", result)
        self.assertIn("version", result)
        self.assertIn("test", result)
        self.assertIn("question", result)
        self.assertIn("person", result)

        self.assertIn("sex", result.get("person"))
        self.assertIn("age", result.get("person"))

        self.assertTrue(len(result.get("id")))
        self.assertTrue(len(result.get("theory")))
        self.assertTrue(len(result.get("model")))

        self.assertEqual(result.get("question"), 120)
        self.assertEqual(result.get("test"), True)
        self.assertEqual(result.get("person").get("sex"), "M")
        self.assertEqual(result.get("person").get("age"), 40)

        self.assertTrue(len(result.get("person").get("result")))
        self.assertTrue(len(result.get("person").get("result").get("personalities")))
        self.assertTrue(len(result.get("person").get("result").get("compare")))

        personalities = result.get("person").get("result").get("personalities")
        self.assertEqual(len(personalities), 5)

        compare = result.get("person").get("result").get("compare")
        self.assertIn("user_answers_original", compare)
        self.assertIn("user_answers_reversed", compare)

        select1 = [
            x.get("reverse_scored") for x in compare.get("user_answers_original", [])
        ]
        assert isinstance(select1, list), "select1 must be a list"

        self.assertEqual(select1.count(0), 67)
        self.assertEqual(select1.count(1), 53)

        select2 = [
            x.get("reverse_scored") for x in compare.get("user_answers_reversed", [])
        ]
        assert isinstance(select2, list), "select2 must be a list"

        self.assertEqual(select2.count(0), select1.count(0))
        self.assertEqual(select2.count(1), select1.count(1))

        self.assertEqual(personalities[0]["Openness"].get("O"), 17.42235805055492)
        self.assertEqual(
            personalities[1]["Conscientiousness"].get("C"), 18.62283542366208
        )
        self.assertEqual(personalities[2]["Extraversion"].get("E"), 34.10779114957646)
        self.assertEqual(personalities[3]["Agreeableness"].get("A"), 6.331832993524202)
        self.assertEqual(personalities[4]["Neuroticism"].get("N"), 58.6632925696303)



"""Unit tests for Facet."""

import json
import unittest

from ipipneo.facet import Facet
from ipipneo.norm import Norm
from ipipneo.utility import organize_list_json


def load_mock_answers_120() -> dict:
    with open("test/mock/answers-test-1.json") as f:
        data = json.load(f)
    return data


def load_mock_answers_300() -> dict:
    with open("test/mock/answers-test-4.json") as f:
        data = json.load(f)
    return data


class TestFacet(unittest.TestCase):
    def test_invalid_params(self) -> None:
        with self.assertRaises(TypeError):
            facet = Facet()

        with self.assertRaises(BaseException):
            facet = Facet(nquestion="")

        with self.assertRaises(BaseException):
            facet = Facet(nquestion=0)

        self.assertEqual(Facet(nquestion=120).__class__.__name__, "Facet")
        self.assertEqual(Facet(nquestion=300).__class__.__name__, "Facet")

    def test_score_120(self) -> None:
        facet = Facet(nquestion=120)

        with self.assertRaises(AttributeError):
            facet.score(answers="")

        with self.assertRaises(TypeError):
            facet.score(answers=["A", "B"])

        with self.assertRaises(BaseException) as e:
            facet.score(answers=[0, 1])
        self.assertEqual(
            str(e.exception),
            "The number of questions setting is wrong: list index out of range",
        )

        score = facet.score(answers=organize_list_json(load_mock_answers_120()))
        assert isinstance(score, list), "score must be a list"
        self.assertEqual(len(score), 121)

    def test_score_300(self) -> None:
        facet = Facet(nquestion=300)

        with self.assertRaises(AttributeError):
            facet.score(answers="")

        with self.assertRaises(TypeError):
            facet.score(answers=["A", "B"])

        with self.assertRaises(BaseException) as e:
            facet.score(answers=[0, 1])
        self.assertEqual(
            str(e.exception),
            "The number of questions setting is wrong: list index out of range",
        )

        with self.assertRaises(BaseException) as e:
            facet.score(answers=organize_list_json(load_mock_answers_120()))
        self.assertEqual(
            str(e.exception),
            "The number of questions setting is wrong: list index out of range",
        )

        score = facet.score(answers=organize_list_json(load_mock_answers_300()))
        assert isinstance(score, list), "score must be a list"
        self.assertEqual(len(score), 301)

    def test_b5create_120(self) -> None:
        facet = Facet(nquestion=120)

        with self.assertRaises(BaseException) as e:
            facet.b5create(score=[0, 0, 0])
        self.assertEqual(
            str(e.exception),
            "The number of questions setting is wrong: list index out of range",
        )

        b5 = facet.b5create(
            score=facet.score(answers=organize_list_json(load_mock_answers_120()))
        )
        assert isinstance(b5, dict), "b5 must be a dict"

        self.assertEqual(len(b5.get("O")), 121)
        self.assertEqual(len(b5.get("C")), 121)
        self.assertEqual(len(b5.get("E")), 121)
        self.assertEqual(len(b5.get("A")), 121)
        self.assertEqual(len(b5.get("N")), 121)

        self.assertEqual(b5.get("O")[0], 0)
        self.assertEqual(b5.get("C")[0], 0)
        self.assertEqual(b5.get("E")[0], 0)
        self.assertEqual(b5.get("A")[0], 0)
        self.assertEqual(b5.get("N")[0], 0)

    def test_b5create_300(self) -> None:
        facet = Facet(nquestion=300)

        with self.assertRaises(BaseException) as e:
            facet.b5create(score=[0, 0, 0])
        self.assertEqual(
            str(e.exception),
            "The number of questions setting is wrong: list index out of range",
        )

        b5 = facet.b5create(
            score=facet.score(answers=organize_list_json(load_mock_answers_300()))
        )
        assert isinstance(b5, dict), "b5 must be a dict"

        self.assertEqual(len(b5.get("O")), 301)
        self.assertEqual(len(b5.get("C")), 301)
        self.assertEqual(len(b5.get("E")), 301)
        self.assertEqual(len(b5.get("A")), 301)
        self.assertEqual(len(b5.get("N")), 301)

        self.assertEqual(b5.get("O")[0], 0)
        self.assertEqual(b5.get("C")[0], 0)
        self.assertEqual(b5.get("E")[0], 0)
        self.assertEqual(b5.get("A")[0], 0)
        self.assertEqual(b5.get("N")[0], 0)

    def test_domain_120(self) -> None:
        facet = Facet(nquestion=120)

        with self.assertRaises(BaseException) as e:
            facet.domain(score=[-1, -1, -1])
        self.assertEqual(
            str(e.exception),
            "Invalid position in the score array: list index out of range",
        )

        domain = facet.domain(
            score=facet.score(answers=organize_list_json(load_mock_answers_120()))
        )
        assert isinstance(domain, dict), "domain must be a dict"

        self.assertEqual(domain.get("O"), 78)
        self.assertEqual(domain.get("C"), 102)
        self.assertEqual(domain.get("E"), 66)
        self.assertEqual(domain.get("A"), 87)
        self.assertEqual(domain.get("N"), 61)

    def test_domain_300(self) -> None:
        facet = Facet(nquestion=300)

        with self.assertRaises(BaseException) as e:
            facet.domain(score=[-1, -1, -1])
        self.assertEqual(
            str(e.exception),
            "Invalid position in the score array: list index out of range",
        )

        domain = facet.domain(
            score=facet.score(answers=organize_list_json(load_mock_answers_300()))
        )
        assert isinstance(domain, dict), "domain must be a dict"

        self.assertEqual(domain.get("O"), 180)
        self.assertEqual(domain.get("C"), 183)
        self.assertEqual(domain.get("E"), 188)
        self.assertEqual(domain.get("A"), 188)
        self.assertEqual(domain.get("N"), 187)

    def test_distrib_120(self) -> None:
        facet = Facet(nquestion=120)

        with self.assertRaises(TypeError):
            facet.distrib(size=0, b5={}, norm={})

        b5 = facet.b5create(
            score=facet.score(answers=organize_list_json(load_mock_answers_120()))
        )
        assert isinstance(b5, dict), "b5 must be a dict"

        domain = facet.domain(
            score=facet.score(answers=organize_list_json(load_mock_answers_120()))
        )
        assert isinstance(domain, dict), "domain must be a dict"

        with self.assertRaises(BaseException) as e:
            facet.distrib(size=0, b5=b5, norm=Norm(sex="M", age=40, nquestion=120))
        self.assertEqual(
            str(e.exception),
            "The number of questions setting is wrong: list assignment index out of range",
        )

        distrib = facet.distrib(
            size=7, b5=b5, norm=Norm(sex="M", age=40, nquestion=120)
        )
        assert isinstance(distrib, dict), "distrib must be a dict"

        self.assertEqual(distrib.get("O")[0], 0)
        self.assertEqual(distrib.get("C")[0], 0)
        self.assertEqual(distrib.get("E")[0], 0)
        self.assertEqual(distrib.get("A")[0], 0)
        self.assertEqual(distrib.get("N")[0], 0)

        self.assertEqual(distrib.get("O")[1], 31.320754716981135)
        self.assertEqual(distrib.get("O")[2], 39.146005509641874)
        self.assertEqual(distrib.get("O")[3], 41.84952978056427)
        self.assertEqual(distrib.get("O")[4], 58.43657817109144)
        self.assertEqual(distrib.get("O")[5], 43.50769230769231)
        self.assertEqual(distrib.get("O")[6], 58.97849462365591)

        self.assertEqual(distrib.get("C")[1], 54.91803278688524)
        self.assertEqual(distrib.get("C")[2], 68.66197183098592)
        self.assertEqual(distrib.get("C")[3], 51.15942028985507)
        self.assertEqual(distrib.get("C")[4], 60.73746312684366)
        self.assertEqual(distrib.get("C")[5], 64.29003021148037)
        self.assertEqual(distrib.get("C")[6], 46.749379652605455)

        self.assertEqual(distrib.get("E")[1], 41.28947368421052)
        self.assertEqual(distrib.get("E")[2], 36.61654135338346)
        self.assertEqual(distrib.get("E")[3], 42.79329608938548)
        self.assertEqual(distrib.get("E")[4], 55.975232198142415)
        self.assertEqual(distrib.get("E")[5], 46.0932944606414)
        self.assertEqual(distrib.get("E")[6], 43.48441926345609)

        self.assertEqual(distrib.get("A")[1], 55.9349593495935)
        self.assertEqual(distrib.get("A")[2], 62.79874213836478)
        self.assertEqual(distrib.get("A")[3], 41.88356164383562)
        self.assertEqual(distrib.get("A")[4], 53.83783783783784)
        self.assertEqual(distrib.get("A")[5], 51.596638655462186)
        self.assertEqual(distrib.get("A")[6], 41.58054711246201)

        self.assertEqual(distrib.get("N")[1], 51.48936170212766)
        self.assertEqual(distrib.get("N")[2], 50.58139534883721)
        self.assertEqual(distrib.get("N")[3], 44.247572815533985)
        self.assertEqual(distrib.get("N")[4], 47.08661417322835)
        self.assertEqual(distrib.get("N")[5], 43.80681818181818)
        self.assertEqual(distrib.get("N")[6], 46.75287356321839)

        distrib = facet.distrib(
            size=7, b5=b5, norm=Norm(sex="F", age=30, nquestion=120)
        )
        assert isinstance(distrib, dict), "distrib must be a dict"

        self.assertEqual(distrib.get("O")[1], 33.113772455089816)
        self.assertEqual(distrib.get("O")[2], 34.93939393939394)
        self.assertEqual(distrib.get("O")[3], 33.605947955390334)
        self.assertEqual(distrib.get("O")[4], 59.18604651162791)
        self.assertEqual(distrib.get("O")[5], 46.31123919308357)
        self.assertEqual(distrib.get("O")[6], 58.49710982658959)

        self.assertEqual(distrib.get("C")[1], 54.18803418803419)
        self.assertEqual(distrib.get("C")[2], 66.25277161862527)
        self.assertEqual(distrib.get("C")[3], 48.58267716535433)
        self.assertEqual(distrib.get("C")[4], 59.47540983606558)
        self.assertEqual(distrib.get("C")[5], 63.74613003095975)
        self.assertEqual(distrib.get("C")[6], 47.822966507177036)

        self.assertEqual(distrib.get("E")[1], 38.9344262295082)
        self.assertEqual(distrib.get("E")[2], 35.382716049382715)
        self.assertEqual(distrib.get("E")[3], 43.767313019390585)
        self.assertEqual(distrib.get("E")[4], 53.7962962962963)
        self.assertEqual(distrib.get("E")[5], 47.62039660056657)
        self.assertEqual(distrib.get("E")[6], 41.05740181268882)

        self.assertEqual(distrib.get("A")[1], 54.98652291105121)
        self.assertEqual(distrib.get("A")[2], 60.191570881226056)
        self.assertEqual(distrib.get("A")[3], 34.940711462450594)
        self.assertEqual(distrib.get("A")[4], 51.22857142857143)
        self.assertEqual(distrib.get("A")[5], 47.25490196078431)
        self.assertEqual(distrib.get("A")[6], 34.250871080139376)

        self.assertEqual(distrib.get("N")[1], 47.065217391304344)
        self.assertEqual(distrib.get("N")[2], 48.256658595641646)
        self.assertEqual(distrib.get("N")[3], 43.14496314496315)
        self.assertEqual(distrib.get("N")[4], 46.83377308707124)
        self.assertEqual(distrib.get("N")[5], 42.43016759776536)
        self.assertEqual(distrib.get("N")[6], 42.60989010989011)

    def test_distrib_300(self) -> None:
        facet = Facet(nquestion=300)

        with self.assertRaises(TypeError):
            facet.distrib(size=0, b5={}, norm={})

        b5 = facet.b5create(
            score=facet.score(answers=organize_list_json(load_mock_answers_300()))
        )
        assert isinstance(b5, dict), "b5 must be a dict"

        domain = facet.domain(
            score=facet.score(answers=organize_list_json(load_mock_answers_300()))
        )
        assert isinstance(domain, dict), "domain must be a dict"

        with self.assertRaises(BaseException) as e:
            facet.distrib(size=0, b5=b5, norm=Norm(sex="M", age=40, nquestion=300))
        self.assertEqual(
            str(e.exception),
            "The number of questions setting is wrong: list assignment index out of range",
        )

        distrib = facet.distrib(
            size=7, b5=b5, norm=Norm(sex="M", age=40, nquestion=300)
        )
        assert isinstance(distrib, dict), "distrib must be a dict"

        self.assertEqual(distrib.get("O")[0], 0)
        self.assertEqual(distrib.get("C")[0], 0)
        self.assertEqual(distrib.get("E")[0], 0)
        self.assertEqual(distrib.get("A")[0], 0)
        self.assertEqual(distrib.get("N")[0], 0)

        self.assertEqual(distrib.get("O")[1], 33.18181818181818)
        self.assertEqual(distrib.get("O")[2], 40.735294117647065)
        self.assertEqual(distrib.get("O")[3], 52.38805970149254)
        self.assertEqual(distrib.get("O")[4], 41.21212121212122)
        self.assertEqual(distrib.get("O")[5], 32.23880597014926)
        self.assertEqual(distrib.get("O")[6], 44.074074074074076)

        self.assertEqual(distrib.get("C")[1], 39.99999999999999)
        self.assertEqual(distrib.get("C")[2], 46.15384615384615)
        self.assertEqual(distrib.get("C")[3], 22.931034482758616)
        self.assertEqual(distrib.get("C")[4], 34.776119402985074)
        self.assertEqual(distrib.get("C")[5], 49.64285714285714)
        self.assertEqual(distrib.get("C")[6], 53.561643835616444)

        self.assertEqual(distrib.get("E")[1], 49.876543209876544)
        self.assertEqual(distrib.get("E")[2], 52.098765432098766)
        self.assertEqual(distrib.get("E")[3], 49.199999999999996)
        self.assertEqual(distrib.get("E")[4], 50.666666666666664)
        self.assertEqual(distrib.get("E")[5], 45.0)
        self.assertEqual(distrib.get("E")[6], 46.57142857142858)

        self.assertEqual(distrib.get("A")[1], 51.84210526315789)
        self.assertEqual(distrib.get("A")[2], 40.3125)
        self.assertEqual(distrib.get("A")[3], 44.375)
        self.assertEqual(distrib.get("A")[4], 51.884057971014485)
        self.assertEqual(distrib.get("A")[5], 56.52173913043478)
        self.assertEqual(distrib.get("A")[6], 32.05882352941176)

        self.assertEqual(distrib.get("N")[1], 51.75)
        self.assertEqual(distrib.get("N")[2], 50.0)
        self.assertEqual(distrib.get("N")[3], 57.55102040816326)
        self.assertEqual(distrib.get("N")[4], 52.89473684210526)
        self.assertEqual(distrib.get("N")[5], 51.23287671232877)
        self.assertEqual(distrib.get("N")[6], 68.10810810810811)

    def test_personality_120(self) -> None:
        facet = Facet(nquestion=120)

        with self.assertRaises(BaseException) as e:
            facet.personality(size=1, big5={}, traits={}, label="")
        self.assertEqual(
            str(e.exception),
            "The Big-Five label is invalid!",
        )

        with self.assertRaises(BaseException) as e:
            facet.personality(size=1, big5={}, traits={}, label="OCEAN")
        self.assertEqual(
            str(e.exception),
            "The Big-Five label is invalid!",
        )

        with self.assertRaises(BaseException) as e:
            facet.personality(size=1, big5={}, traits={}, label="N")
        self.assertEqual(
            str(e.exception),
            "The number of questions setting is wrong: list index out of range",
        )

        score = facet.score(answers=organize_list_json(load_mock_answers_120()))
        assert isinstance(score, list), "distrib must be a list"
        self.assertEqual(len(score), 121)

        b5 = facet.b5create(score=score)
        assert isinstance(b5, dict), "b5 must be a dict"

        domain = facet.domain(score=score)
        assert isinstance(domain, dict), "domain must be a dict"

        norm = Norm(sex="M", age=40, nquestion=120)
        assert isinstance(norm, dict), "norm must be a dict"

        normc = Norm.calc(domain=domain, norm=norm)
        assert isinstance(normc, dict), "normc must be a dict"

        distrib = facet.distrib(size=len(score), b5=b5, norm=norm)
        assert isinstance(distrib, dict), "distrib must be a dict"

        percent = Norm.percent(normc=normc)
        assert isinstance(percent, dict), "percent must be a dict"

        normalize = Norm.normalize(normc=normc, percent=percent)
        assert isinstance(normalize, dict), "normalize must be a dict"

        #############################################
        # 1. Neuroticism with its facets.
        #############################################
        N = facet.personality(
            size=len(score), big5=normalize, traits=distrib, label="N"
        )
        assert isinstance(N, dict), "N must be a dict"
        assert isinstance(N.get("traits"), list), "traits must be a list"

        self.assertEqual(N.get("N"), 37.23631660596993)

        self.assertEqual(N.get("traits")[0]["trait"], 1)
        self.assertEqual(N.get("traits")[0]["Anxiety"], 55.29153291804727)
        self.assertEqual(N.get("traits")[0]["score"], "average")

        self.assertEqual(N.get("traits")[1]["trait"], 2)
        self.assertEqual(N.get("traits")[1]["Anger"], 52.068589887460575)
        self.assertEqual(N.get("traits")[1]["score"], "average")

        self.assertEqual(N.get("traits")[2]["trait"], 3)
        self.assertEqual(N.get("traits")[2]["Depression"], 30.04288206625688)
        self.assertEqual(N.get("traits")[2]["score"], "low")

        self.assertEqual(N.get("traits")[3]["trait"], 4)
        self.assertEqual(N.get("traits")[3]["Self-Consciousness"], 39.69850290396829)
        self.assertEqual(N.get("traits")[3]["score"], "average")

        self.assertEqual(N.get("traits")[4]["trait"], 5)
        self.assertEqual(N.get("traits")[4]["Immoderation"], 28.601999121511483)
        self.assertEqual(N.get("traits")[4]["score"], "low")

        self.assertEqual(N.get("traits")[5]["trait"], 6)
        self.assertEqual(N.get("traits")[5]["Vulnerability"], 38.536490293892996)
        self.assertEqual(N.get("traits")[5]["score"], "average")

        #############################################
        # 2. Extraversion with its facets.
        #############################################
        E = facet.personality(
            size=len(score), big5=normalize, traits=distrib, label="E"
        )
        assert isinstance(N, dict), "E must be a dict"
        assert isinstance(N.get("traits"), list), "traits must be a list"

        self.assertEqual(E.get("E"), 21.46713047820697)

        self.assertEqual(E.get("traits")[0]["trait"], 1)
        self.assertEqual(E.get("traits")[0]["Friendliness"], 20.788746198012063)
        self.assertEqual(E.get("traits")[0]["score"], "low")

        self.assertEqual(E.get("traits")[1]["trait"], 2)
        self.assertEqual(E.get("traits")[1]["Gregariousness"], 8.857170154068399)
        self.assertEqual(E.get("traits")[1]["score"], "low")

        self.assertEqual(E.get("traits")[2]["trait"], 3)
        self.assertEqual(E.get("traits")[2]["Assertiveness"], 25.365062122504668)
        self.assertEqual(E.get("traits")[2]["score"], "low")

        self.assertEqual(E.get("traits")[3]["trait"], 4)
        self.assertEqual(E.get("traits")[3]["Activity-Level"], 70.68785093930268)
        self.assertEqual(E.get("traits")[3]["score"], "high")

        self.assertEqual(E.get("traits")[4]["trait"], 5)
        self.assertEqual(E.get("traits")[4]["Excitement-Seeking"], 36.25782829248527)
        self.assertEqual(E.get("traits")[4]["score"], "average")

        self.assertEqual(E.get("traits")[5]["trait"], 6)
        self.assertEqual(E.get("traits")[5]["Cheerfulness"], 27.560328329400278)
        self.assertEqual(E.get("traits")[5]["score"], "low")

        #############################################
        # 3. Openness with its facets.
        #############################################
        O = facet.personality(
            size=len(score), big5=normalize, traits=distrib, label="O"
        )
        assert isinstance(O, dict), "O must be a dict"
        assert isinstance(O.get("traits"), list), "traits must be a list"

        self.assertEqual(O.get("O"), 26.882298529738506)

        self.assertEqual(O.get("traits")[0]["trait"], 1)
        self.assertEqual(O.get("traits")[0]["Imagination"], 1)
        self.assertEqual(O.get("traits")[0]["score"], "low")

        self.assertEqual(O.get("traits")[1]["trait"], 2)
        self.assertEqual(O.get("traits")[1]["Artistic-Interests"], 14.832308817371768)
        self.assertEqual(O.get("traits")[1]["score"], "low")

        self.assertEqual(O.get("traits")[2]["trait"], 3)
        self.assertEqual(O.get("traits")[2]["Emotionality"], 22.458638690321635)
        self.assertEqual(O.get("traits")[2]["score"], "low")

        self.assertEqual(O.get("traits")[3]["trait"], 4)
        self.assertEqual(O.get("traits")[3]["Adventurousness"], 78.39980163147527)
        self.assertEqual(O.get("traits")[3]["score"], "high")

        self.assertEqual(O.get("traits")[4]["trait"], 5)
        self.assertEqual(O.get("traits")[4]["Intellect"], 27.6351617847169)
        self.assertEqual(O.get("traits")[4]["score"], "low")

        self.assertEqual(O.get("traits")[5]["trait"], 6)
        self.assertEqual(O.get("traits")[5]["Liberalism"], 79.99472618213304)
        self.assertEqual(O.get("traits")[5]["score"], "high")

        #############################################
        # 4. Agreeableness with its facets.
        #############################################
        A = facet.personality(
            size=len(score), big5=normalize, traits=distrib, label="A"
        )
        assert isinstance(A, dict), "A must be a dict"
        assert isinstance(A.get("traits"), list), "traits must be a list"

        self.assertEqual(A.get("A"), 57.52658875205816)

        self.assertEqual(A.get("traits")[0]["trait"], 1)
        self.assertEqual(A.get("traits")[0]["Trust"], 70.5561195632738)
        self.assertEqual(A.get("traits")[0]["score"], "high")

        self.assertEqual(A.get("traits")[1]["trait"], 2)
        self.assertEqual(A.get("traits")[1]["Morality"], 89.87557658724631)
        self.assertEqual(A.get("traits")[1]["score"], "high")

        self.assertEqual(A.get("traits")[2]["trait"], 3)
        self.assertEqual(A.get("traits")[2]["Altruism"], 22.561476501041227)
        self.assertEqual(A.get("traits")[2]["score"], "low")

        self.assertEqual(A.get("traits")[3]["trait"], 4)
        self.assertEqual(A.get("traits")[3]["Cooperation"], 63.50546308998889)
        self.assertEqual(A.get("traits")[3]["score"], "average")

        self.assertEqual(A.get("traits")[4]["trait"], 5)
        self.assertEqual(A.get("traits")[4]["Modesty"], 55.67124528467815)
        self.assertEqual(A.get("traits")[4]["score"], "average")

        self.assertEqual(A.get("traits")[5]["trait"], 6)
        self.assertEqual(A.get("traits")[5]["Sympathy"], 21.651269458084244)
        self.assertEqual(A.get("traits")[5]["score"], "low")

        #############################################
        # 5. Conscientiousness with its facets.
        #############################################
        C = facet.personality(
            size=len(score), big5=normalize, traits=distrib, label="C"
        )
        assert isinstance(C, dict), "C must be a dict"
        assert isinstance(C.get("traits"), list), "traits must be a list"

        self.assertEqual(C.get("C"), 86.96687032595662)

        self.assertEqual(C.get("traits")[0]["trait"], 1)
        self.assertEqual(C.get("traits")[0]["Self-Efficacy"], 67.18081968991703)
        self.assertEqual(C.get("traits")[0]["score"], "average")

        self.assertEqual(C.get("traits")[1]["trait"], 2)
        self.assertEqual(C.get("traits")[1]["Orderliness"], 98.82696217870512)
        self.assertEqual(C.get("traits")[1]["score"], "high")

        self.assertEqual(C.get("traits")[2]["trait"], 3)
        self.assertEqual(C.get("traits")[2]["Dutifulness"], 54.122030713014226)
        self.assertEqual(C.get("traits")[2]["score"], "average")

        self.assertEqual(C.get("traits")[3]["trait"], 4)
        self.assertEqual(C.get("traits")[3]["Achievement-Striving"], 84.86323430898972)
        self.assertEqual(C.get("traits")[3]["score"], "high")

        self.assertEqual(C.get("traits")[4]["trait"], 5)
        self.assertEqual(C.get("traits")[4]["Self-Discipline"], 92.95956099997227)
        self.assertEqual(C.get("traits")[4]["score"], "high")

        self.assertEqual(C.get("traits")[5]["trait"], 6)
        self.assertEqual(C.get("traits")[5]["Cautiousness"], 38.524355262005315)
        self.assertEqual(C.get("traits")[5]["score"], "average")

    def test_personality_300(self) -> None:
        facet = Facet(nquestion=300)

        with self.assertRaises(BaseException) as e:
            facet.personality(size=1, big5={}, traits={}, label="")
        self.assertEqual(
            str(e.exception),
            "The Big-Five label is invalid!",
        )

        with self.assertRaises(BaseException) as e:
            facet.personality(size=1, big5={}, traits={}, label="OCEAN")
        self.assertEqual(
            str(e.exception),
            "The Big-Five label is invalid!",
        )

        with self.assertRaises(BaseException) as e:
            facet.personality(size=1, big5={}, traits={}, label="N")
        self.assertEqual(
            str(e.exception),
            "The number of questions setting is wrong: list index out of range",
        )

        score = facet.score(answers=organize_list_json(load_mock_answers_300()))
        assert isinstance(score, list), "distrib must be a list"
        self.assertEqual(len(score), 301)

        b5 = facet.b5create(score=score)
        assert isinstance(b5, dict), "b5 must be a dict"

        domain = facet.domain(score=score)
        assert isinstance(domain, dict), "domain must be a dict"

        norm = Norm(sex="M", age=40, nquestion=300)
        assert isinstance(norm, dict), "norm must be a dict"

        normc = Norm.calc(domain=domain, norm=norm)
        assert isinstance(normc, dict), "normc must be a dict"

        distrib = facet.distrib(size=len(score), b5=b5, norm=norm)
        assert isinstance(distrib, dict), "distrib must be a dict"

        percent = Norm.percent(normc=normc)
        assert isinstance(percent, dict), "percent must be a dict"

        normalize = Norm.normalize(normc=normc, percent=percent)
        assert isinstance(normalize, dict), "normalize must be a dict"

        #############################################
        # 1. Neuroticism with its facets.
        #############################################
        N = facet.personality(
            size=len(score), big5=normalize, traits=distrib, label="N"
        )
        assert isinstance(N, dict), "N must be a dict"
        assert isinstance(N.get("traits"), list), "traits must be a list"

        self.assertEqual(N.get("N"), 72.42696555428512)

        self.assertEqual(N.get("traits")[0]["trait"], 1)
        self.assertEqual(N.get("traits")[0]["Anxiety"], 56.21355262825239)
        self.assertEqual(N.get("traits")[0]["score"], "average")

        self.assertEqual(N.get("traits")[1]["trait"], 2)
        self.assertEqual(N.get("traits")[1]["Anger"], 49.99999999999858)
        self.assertEqual(N.get("traits")[1]["score"], "average")

        self.assertEqual(N.get("traits")[2]["trait"], 3)
        self.assertEqual(N.get("traits")[2]["Depression"], 75.7080910490638)
        self.assertEqual(N.get("traits")[2]["score"], "high")

        self.assertEqual(N.get("traits")[3]["trait"], 4)
        self.assertEqual(N.get("traits")[3]["Self-Consciousness"], 60.2364043211619)
        self.assertEqual(N.get("traits")[3]["score"], "average")

        self.assertEqual(N.get("traits")[4]["trait"], 5)
        self.assertEqual(N.get("traits")[4]["Immoderation"], 54.38260045296113)
        self.assertEqual(N.get("traits")[4]["score"], "average")

        self.assertEqual(N.get("traits")[5]["trait"], 6)
        self.assertEqual(N.get("traits")[5]["Vulnerability"], 98.37585552257963)
        self.assertEqual(N.get("traits")[5]["score"], "high")

        #############################################
        # 2. Extraversion with its facets.
        #############################################
        E = facet.personality(
            size=len(score), big5=normalize, traits=distrib, label="E"
        )
        assert isinstance(N, dict), "E must be a dict"
        assert isinstance(N.get("traits"), list), "traits must be a list"

        self.assertEqual(E.get("E"), 44.65415108872236)

        self.assertEqual(E.get("traits")[0]["trait"], 1)
        self.assertEqual(E.get("traits")[0]["Friendliness"], 49.56063603396092)
        self.assertEqual(E.get("traits")[0]["score"], "average")

        self.assertEqual(E.get("traits")[1]["trait"], 2)
        self.assertEqual(E.get("traits")[1]["Gregariousness"], 57.44425564700225)
        self.assertEqual(E.get("traits")[1]["score"], "average")

        self.assertEqual(E.get("traits")[2]["trait"], 3)
        self.assertEqual(E.get("traits")[2]["Assertiveness"], 47.15427409876639)
        self.assertEqual(E.get("traits")[2]["score"], "average")

        self.assertEqual(E.get("traits")[3]["trait"], 4)
        self.assertEqual(E.get("traits")[3]["Activity-Level"], 52.371791064982006)
        self.assertEqual(E.get("traits")[3]["score"], "average")

        self.assertEqual(E.get("traits")[4]["trait"], 5)
        self.assertEqual(E.get("traits")[4]["Excitement-Seeking"], 32.54383356563801)
        self.assertEqual(E.get("traits")[4]["score"], "average")

        self.assertEqual(E.get("traits")[5]["trait"], 6)
        self.assertEqual(E.get("traits")[5]["Cheerfulness"], 37.907163869258454)
        self.assertEqual(E.get("traits")[5]["score"], "average")

        #############################################
        # 3. Openness with its facets.
        #############################################
        O = facet.personality(
            size=len(score), big5=normalize, traits=distrib, label="O"
        )
        assert isinstance(O, dict), "O must be a dict"
        assert isinstance(O.get("traits"), list), "traits must be a list"

        self.assertEqual(O.get("O"), 6.765261759154043)

        self.assertEqual(O.get("traits")[0]["trait"], 1)
        self.assertEqual(O.get("traits")[0]["Imagination"], 3.019669333842657)
        self.assertEqual(O.get("traits")[0]["score"], "low")

        self.assertEqual(O.get("traits")[1]["trait"], 2)
        self.assertEqual(O.get("traits")[1]["Artistic-Interests"], 19.180033328925163)
        self.assertEqual(O.get("traits")[1]["score"], "low")

        self.assertEqual(O.get("traits")[2]["trait"], 3)
        self.assertEqual(O.get("traits")[2]["Emotionality"], 58.46198478449537)
        self.assertEqual(O.get("traits")[2]["score"], "average")

        self.assertEqual(O.get("traits")[3]["trait"], 4)
        self.assertEqual(O.get("traits")[3]["Adventurousness"], 20.561530401874762)
        self.assertEqual(O.get("traits")[3]["score"], "low")

        self.assertEqual(O.get("traits")[4]["trait"], 5)
        self.assertEqual(O.get("traits")[4]["Intellect"], 1.9528124319700737)
        self.assertEqual(O.get("traits")[4]["score"], "low")

        self.assertEqual(O.get("traits")[5]["trait"], 6)
        self.assertEqual(O.get("traits")[5]["Liberalism"], 29.473450015379456)
        self.assertEqual(O.get("traits")[5]["score"], "low")

        #############################################
        # 4. Agreeableness with its facets.
        #############################################
        A = facet.personality(
            size=len(score), big5=normalize, traits=distrib, label="A"
        )
        assert isinstance(A, dict), "A must be a dict"
        assert isinstance(A.get("traits"), list), "traits must be a list"

        self.assertEqual(A.get("A"), 31.13757270081055)

        self.assertEqual(A.get("traits")[0]["trait"], 1)
        self.assertEqual(A.get("traits")[0]["Trust"], 56.53893235643574)
        self.assertEqual(A.get("traits")[0]["score"], "average")

        self.assertEqual(A.get("traits")[1]["trait"], 2)
        self.assertEqual(A.get("traits")[1]["Morality"], 17.983638162857602)
        self.assertEqual(A.get("traits")[1]["score"], "low")

        self.assertEqual(A.get("traits")[2]["trait"], 3)
        self.assertEqual(A.get("traits")[2]["Altruism"], 30.46290046715066)
        self.assertEqual(A.get("traits")[2]["score"], "low")

        self.assertEqual(A.get("traits")[3]["trait"], 4)
        self.assertEqual(A.get("traits")[3]["Cooperation"], 56.687055095672974)
        self.assertEqual(A.get("traits")[3]["score"], "average")

        self.assertEqual(A.get("traits")[4]["trait"], 5)
        self.assertEqual(A.get("traits")[4]["Modesty"], 72.45946413723033)
        self.assertEqual(A.get("traits")[4]["score"], "high")

        self.assertEqual(A.get("traits")[5]["trait"], 6)
        self.assertEqual(A.get("traits")[5]["Sympathy"], 1.7779210136593235)
        self.assertEqual(A.get("traits")[5]["score"], "low")

        #############################################
        # 5. Conscientiousness with its facets.
        #############################################
        C = facet.personality(
            size=len(score), big5=normalize, traits=distrib, label="C"
        )
        assert isinstance(C, dict), "C must be a dict"
        assert isinstance(C.get("traits"), list), "traits must be a list"

        self.assertEqual(C.get("C"), 15.799085796472156)

        self.assertEqual(C.get("traits")[0]["trait"], 1)
        self.assertEqual(C.get("traits")[0]["Self-Efficacy"], 17.11734969494421)
        self.assertEqual(C.get("traits")[0]["score"], "low")

        self.assertEqual(C.get("traits")[1]["trait"], 2)
        self.assertEqual(C.get("traits")[1]["Orderliness"], 36.46593774743093)
        self.assertEqual(C.get("traits")[1]["score"], "average")

        self.assertEqual(C.get("traits")[2]["trait"], 3)
        self.assertEqual(C.get("traits")[2]["Dutifulness"], 1)
        self.assertEqual(C.get("traits")[2]["score"], "low")

        self.assertEqual(C.get("traits")[3]["trait"], 4)
        self.assertEqual(C.get("traits")[3]["Achievement-Striving"], 5.368564900396208)
        self.assertEqual(C.get("traits")[3]["score"], "low")

        self.assertEqual(C.get("traits")[4]["trait"], 5)
        self.assertEqual(C.get("traits")[4]["Self-Discipline"], 48.729091361349106)
        self.assertEqual(C.get("traits")[4]["score"], "average")

        self.assertEqual(C.get("traits")[5]["trait"], 6)
        self.assertEqual(C.get("traits")[5]["Cautiousness"], 62.553226842568904)
        self.assertEqual(C.get("traits")[5]["score"], "average")



"""Unit tests for Model."""

import unittest

from ipipneo.model import (Big5Agreeableness, Big5Conscientiousness,
                           Big5Extraversion, Big5Neuroticism, Big5Openness,
                           FacetLevel, FacetScale, NormCubic, NormScale,
                           QuestionNumber)


class TestModel(unittest.TestCase):
    def test_question_number(self) -> None:
        model = QuestionNumber

        self.assertEqual(model.IPIP_120, 120)
        self.assertEqual(model.IPIP_120.value, 120)

        self.assertEqual(model.IPIP_300, 300)
        self.assertEqual(model.IPIP_300.value, 300)

        x1 = list(map(int, model))
        self.assertEqual(len(x1), 2)

    def test_facet_scale(self) -> None:
        model = FacetScale

        self.assertEqual(model.IPIP_MAX, 30)
        self.assertEqual(model.IPIP_MAX.value, 30)

        self.assertEqual(model.IPIP_300, 10)
        self.assertEqual(model.IPIP_300.value, 10)

        self.assertEqual(model.IPIP_120, 4)
        self.assertEqual(model.IPIP_120.value, 4)

        x1 = list(map(int, model))
        self.assertEqual(len(x1), 3)

    def test_facet_level(self) -> None:
        model = FacetLevel

        self.assertEqual(model.HIGH, 55)
        self.assertEqual(model.HIGH.value, 55)

        self.assertEqual(model.LOW, 45)
        self.assertEqual(model.LOW.value, 45)

        x1 = list(map(int, model))
        self.assertEqual(len(x1), 2)

    def test_norm_scale(self) -> None:
        model = NormScale

        self.assertEqual(model.CONST_MAX, 73)
        self.assertEqual(model.CONST_MAX.value, 73)

        self.assertEqual(model.CONST_MIN, 32)
        self.assertEqual(model.CONST_MIN.value, 32)

        x1 = list(map(int, model))
        self.assertEqual(len(x1), 2)

    def test_norm_cubic(self) -> None:
        model = NormCubic

        self.assertEqual(model.CONST1, 210.335958661391)
        self.assertEqual(model.CONST1.value, 210.335958661391)

        self.assertEqual(model.CONST2, 16.7379362643389)
        self.assertEqual(model.CONST2.value, 16.7379362643389)

        self.assertEqual(model.CONST3, 0.405936512733332)
        self.assertEqual(model.CONST3.value, 0.405936512733332)

        self.assertEqual(model.CONST4, 0.00270624341822222)
        self.assertEqual(model.CONST4.value, 0.00270624341822222)

        x1 = list(map(int, model))
        self.assertEqual(len(x1), 4)

    def test_big5_neuroticism(self) -> None:
        model = Big5Neuroticism

        self.assertEqual(model.TRAIT1, "Anxiety")
        self.assertEqual(model.TRAIT1.value, "Anxiety")

        self.assertEqual(model.TRAIT2, "Anger")
        self.assertEqual(model.TRAIT2.value, "Anger")

        self.assertEqual(model.TRAIT3, "Depression")
        self.assertEqual(model.TRAIT3.value, "Depression")

        self.assertEqual(model.TRAIT4, "Self-Consciousness")
        self.assertEqual(model.TRAIT4.value, "Self-Consciousness")

        self.assertEqual(model.TRAIT5, "Immoderation")
        self.assertEqual(model.TRAIT5.value, "Immoderation")

        self.assertEqual(model.TRAIT6, "Vulnerability")
        self.assertEqual(model.TRAIT6.value, "Vulnerability")

        x1 = list(map(str, model))
        self.assertEqual(len(x1), 6)

    def test_big5_extraversion(self) -> None:
        model = Big5Extraversion

        self.assertEqual(model.TRAIT1, "Friendliness")
        self.assertEqual(model.TRAIT1.value, "Friendliness")

        self.assertEqual(model.TRAIT2, "Gregariousness")
        self.assertEqual(model.TRAIT2.value, "Gregariousness")

        self.assertEqual(model.TRAIT3, "Assertiveness")
        self.assertEqual(model.TRAIT3.value, "Assertiveness")

        self.assertEqual(model.TRAIT4, "Activity-Level")
        self.assertEqual(model.TRAIT4.value, "Activity-Level")

        self.assertEqual(model.TRAIT5, "Excitement-Seeking")
        self.assertEqual(model.TRAIT5.value, "Excitement-Seeking")

        self.assertEqual(model.TRAIT6, "Cheerfulness")
        self.assertEqual(model.TRAIT6.value, "Cheerfulness")

        x1 = list(map(str, model))
        self.assertEqual(len(x1), 6)

    def test_big5_openness(self) -> None:
        model = Big5Openness

        self.assertEqual(model.TRAIT1, "Imagination")
        self.assertEqual(model.TRAIT1.value, "Imagination")

        self.assertEqual(model.TRAIT2, "Artistic-Interests")
        self.assertEqual(model.TRAIT2.value, "Artistic-Interests")

        self.assertEqual(model.TRAIT3, "Emotionality")
        self.assertEqual(model.TRAIT3.value, "Emotionality")

        self.assertEqual(model.TRAIT4, "Adventurousness")
        self.assertEqual(model.TRAIT4.value, "Adventurousness")

        self.assertEqual(model.TRAIT5, "Intellect")
        self.assertEqual(model.TRAIT5.value, "Intellect")

        self.assertEqual(model.TRAIT6, "Liberalism")
        self.assertEqual(model.TRAIT6.value, "Liberalism")

        x1 = list(map(str, model))
        self.assertEqual(len(x1), 6)

    def test_big5_agreeableness(self) -> None:
        model = Big5Agreeableness

        self.assertEqual(model.TRAIT1, "Trust")
        self.assertEqual(model.TRAIT1.value, "Trust")

        self.assertEqual(model.TRAIT2, "Morality")
        self.assertEqual(model.TRAIT2.value, "Morality")

        self.assertEqual(model.TRAIT3, "Altruism")
        self.assertEqual(model.TRAIT3.value, "Altruism")

        self.assertEqual(model.TRAIT4, "Cooperation")
        self.assertEqual(model.TRAIT4.value, "Cooperation")

        self.assertEqual(model.TRAIT5, "Modesty")
        self.assertEqual(model.TRAIT5.value, "Modesty")

        self.assertEqual(model.TRAIT6, "Sympathy")
        self.assertEqual(model.TRAIT6.value, "Sympathy")

        x1 = list(map(str, model))
        self.assertEqual(len(x1), 6)

    def test_big5_conscientiousness(self) -> None:
        model = Big5Conscientiousness

        self.assertEqual(model.TRAIT1, "Self-Efficacy")
        self.assertEqual(model.TRAIT1.value, "Self-Efficacy")

        self.assertEqual(model.TRAIT2, "Orderliness")
        self.assertEqual(model.TRAIT2.value, "Orderliness")

        self.assertEqual(model.TRAIT3, "Dutifulness")
        self.assertEqual(model.TRAIT3.value, "Dutifulness")

        self.assertEqual(model.TRAIT4, "Achievement-Striving")
        self.assertEqual(model.TRAIT4.value, "Achievement-Striving")

        self.assertEqual(model.TRAIT5, "Self-Discipline")
        self.assertEqual(model.TRAIT5.value, "Self-Discipline")

        self.assertEqual(model.TRAIT6, "Cautiousness")
        self.assertEqual(model.TRAIT6.value, "Cautiousness")

        x1 = list(map(str, model))
        self.assertEqual(len(x1), 6)



"""Contains utility functions used in the library."""

__author__ = "Ederson Corbari"
__email__ = "e@neural7.io"
__copyright__ = "Copyright Neural7 2022, Big 5 Personality Traits"
__credits__ = ["John A. Johnson", "Dhiru Kholia"]
__license__ = "MIT"
__version__ = "1.0.0"
__status__ = "production"

from enum import Enum

from ipipneo.model import (Big5Agreeableness, Big5Conscientiousness,
                           Big5Extraversion, Big5Neuroticism, Big5Openness)


def raise_if_sex_is_invalid(sex: str) -> bool or AssertionError or BaseException:
    """
    Validate the sex field.

    Args:
        - sex: Male is M or Female is F.
    """
    if not sex:
        raise BaseException("The (sex) field is required!")

    assert isinstance(sex, str), "The (sex) field must be a string!"
    assert sex == "M" or sex == "F", "The (sex) field must contain (M or F)!"
    return True


def raise_if_age_is_invalid(age: int) -> bool or AssertionError or BaseException:
    """
    Validate the age field.

    Args:
        - age: Person's age.
    """
    if not age:
        raise BaseException("The (age) field is required!")

    assert isinstance(age, int), "The (age) field must be an integer!"

    min, max = (18, 100)
    if not (min <= age <= max):
        raise AssertionError(
            "The age (%r) must be between %r and %r!" % (age, min, max)
        )

    return True


def answers_is_valid(answers: list) -> bool or AssertionError or BaseException:
    """
    Validate the answers field.

    Args:
        - answers: List with the answers.
    """
    if not answers:
        raise BaseException("The (answers) field is required!")

    assert isinstance(answers, list), "The (answers) field must be a list!"

    if 0 in answers:
        raise BaseException("It cannot contain zeros in the answer list!")

    if 1 in [1 if x > 5 else 0 for x in answers]:
        raise BaseException("You cannot have answers with a number greater than 5!")

    assert (
        len(answers) == 120 or len(answers) == 300
    ), "The (answers) field should be of size 120 or 300!"

    return True


def organize_list_json(answers: dict) -> list or AssertionError or BaseException:
    """
    Organize input list in json format.

    Args:
        - answers: Dictionary with the list of answers.
    """
    assert isinstance(answers, dict), "The (answers) field must be a dict!"

    if "answers" not in answers:
        raise BaseException("The key named (answers) was not found!")

    if not any("id_question" in x for x in answers.get("answers", [])):
        raise BaseException("The key named (id_question) was not found!")

    if not any("id_select" in x for x in answers.get("answers", [])):
        raise BaseException("The key named (id_select) was not found!")

    return [
        x["id_select"]
        for x in sorted(
            [x for x in answers.get("answers", [])], key=lambda x: x["id_question"]
        )
        if x["id_select"] >= 1
    ]


def reverse_scored(select: int) -> int or BaseException:
    """
    Apply reverse scoring (1=5, 2=4, 4=2, 5=1).

    Args:
        - answers: Selected item.
    """
    if select == 1:
        return 5
    elif select == 2:
        return 4
    elif select == 3:
        return 3
    elif select == 4:
        return 2
    elif select == 5:
        return 1
    raise BaseException(f"Something wrong in the selection option: {select}")


def big5_ocean_is_valid(label: str) -> bool or BaseException:
    """
    Validate if the Big-Five acronym is correct.

    Args:
        - label: The acronym for the Big-Five standard O.C.E.A.N.
    """
    if label == "O":
        return True
    if label == "C":
        return True
    if label == "E":
        return True
    if label == "A":
        return True
    if label == "N":
        return True
    raise BaseException("The Big-Five label is invalid!")


def big5_target(label: str) -> Enum or BaseException:
    """
    Return the facets of a Big-Five.

    Args:
        - label: The acronym for the Big-Five standard O.C.E.A.N.
    """
    if big5_ocean_is_valid(label=label):
        if label == "O":
            return Big5Openness
        if label == "C":
            return Big5Conscientiousness
        if label == "E":
            return Big5Extraversion
        if label == "A":
            return Big5Agreeableness
        if label == "N":
            return Big5Neuroticism


def create_big5_dict(label: str, big5: float, x: list, y: list) -> dict:
    """
    Create a dictionary for Big-Five final result.

    Args:
        - label: The acronym for the Big-Five standard O.C.E.A.N.
        - big5: The list with the personalities / facets.
        - x: The list with the facets.
        - y: The list of scores.
    """
    return {
        label: big5,
        "traits": [
            {
                "trait": 1,
                big5_target(label=label).TRAIT1.value: x[1],
                "score": y[1],
            },
            {
                "trait": 2,
                big5_target(label=label).TRAIT2.value: x[2],
                "score": y[2],
            },
            {
                "trait": 3,
                big5_target(label=label).TRAIT3.value: x[3],
                "score": y[3],
            },
            {
                "trait": 4,
                big5_target(label=label).TRAIT4.value: x[4],
                "score": y[4],
            },
            {
                "trait": 5,
                big5_target(label=label).TRAIT5.value: x[5],
                "score": y[5],
            },
            {
                "trait": 6,
                big5_target(label=label).TRAIT6.value: x[6],
                "score": y[6],
            },
        ],
    }



"""It does the calculations to generate the IPIP-NEO results based on the questions and answers."""

__author__ = "Ederson Corbari"
__email__ = "e@neural7.io"
__copyright__ = "Copyright Neural7 2022, Big 5 Personality Traits"
__credits__ = ["John A. Johnson", "Dhiru Kholia"]
__license__ = "MIT"
__version__ = "1.5.0"
__status__ = "production"

import copy
import uuid

from ipipneo.facet import Facet
from ipipneo.model import QuestionNumber
from ipipneo.norm import Norm
from ipipneo.reverse import (ReverseScored120, ReverseScored300,
                             ReverseScoredCustom)
from ipipneo.utility import (organize_list_json, raise_if_age_is_invalid,
                             raise_if_sex_is_invalid)


class IpipNeo(Facet):
    """Class that calculates IPIP-NEO answers."""

    def __init__(self, question: int, test: bool = False) -> None or BaseException:
        """
        Initialize the class.

        Args:
            - question: Question type, 120 or 300.
            - test: Used to test your proposed questions with reverse.
        """
        if question == 120:
            nquestion = QuestionNumber.IPIP_120
        elif question == 300:
            nquestion = QuestionNumber.IPIP_300
        else:
            raise BaseException(f"Type question {question} is invalid!")

        super().__init__(nquestion=nquestion)
        self._nquestion, self._test = (question, test)

    def evaluator(self, sex: str, age: int, score: list) -> dict:
        """
        Apply the calculation of the Big-Five and its personalities based on the answers.

        Args:
            - sex: Gender of the individual (M or F).
            - age: The age of the individual.
            - score: The normalized score.
        """
        norm = Norm(sex=sex, age=age, nquestion=self._nquestion)
        assert isinstance(norm, dict), "norm must be a dict"

        normc = Norm.calc(domain=self.domain(score=score), norm=norm)
        assert isinstance(normc, dict), "normc must be a dict"

        distrib = self.distrib(
            size=len(score), b5=self.b5create(score=score), norm=norm
        )
        assert isinstance(distrib, dict), "distrib must be a dict"

        normalize = Norm.normalize(normc=normc, percent=Norm.percent(normc=normc))
        assert isinstance(normalize, dict), "normalize must be a dict"

        N = self.personality(size=len(score), big5=normalize, traits=distrib, label="N")
        E = self.personality(size=len(score), big5=normalize, traits=distrib, label="E")
        O = self.personality(size=len(score), big5=normalize, traits=distrib, label="O")
        A = self.personality(size=len(score), big5=normalize, traits=distrib, label="A")
        C = self.personality(size=len(score), big5=normalize, traits=distrib, label="C")

        assert isinstance(O, dict), "O must be a dict"
        assert isinstance(C, dict), "C must be a dict"
        assert isinstance(E, dict), "E must be a dict"
        assert isinstance(A, dict), "A must be a dict"
        assert isinstance(N, dict), "N must be a dict"

        return {
            "id": str(uuid.uuid4()),
            "theory": "Big 5 Personality Traits",
            "model": "IPIP-NEO",
            "question": self._nquestion,
            "test": self._test,
            "version": __version__,
            "person": {
                "sex": sex,
                "age": age,
                "result": {
                    "personalities": [
                        {"Openness": O},
                        {"Conscientiousness": C},
                        {"Extraversion": E},
                        {"Agreeableness": A},
                        {"Neuroticism": N},
                    ]
                },
            },
        }

    def compute(self, sex: str, age: int, answers: dict, compare: bool = False) -> dict:
        """
        Compute the answers and generate the data with the results.

        Args:
            - sex: Gender of the individual (M or F).
            - age: The age of the individual.
            - answers: Standardized dictionary with answers.
            - compare: If true, it shows the user's answers and reverse score.
        """
        raise_if_sex_is_invalid(sex=sex)
        raise_if_age_is_invalid(age=age)
        assert isinstance(answers, dict), "answers must be a dict"

        original = copy.deepcopy(answers)
        assert isinstance(original, dict), "original must be a dict"

        reversed = {}
        if self._test:
            reversed = ReverseScoredCustom(answers=answers)
        else:
            reversed = (
                ReverseScored120(answers=answers)
                if self._nquestion == 120
                else ReverseScored300(answers=answers)
            )
        assert isinstance(reversed, dict), "reversed must be a dict"

        score = self.score(answers=organize_list_json(answers=reversed))
        assert isinstance(score, list), "score must be a list"

        result = self.evaluator(sex=sex, age=age, score=score)
        assert isinstance(result, dict), "result 1 must be a dict"

        if compare:
            result["person"]["result"]["compare"] = {
                "user_answers_original": original.get("answers", []),
                "user_answers_reversed": reversed.get("answers", []),
            }
        assert isinstance(result, dict), "result 2 must be a dict"

        return result or {}



"""Unit tests for Utility."""

import json
import unittest

from ipipneo.reverse import (ReverseScored120, ReverseScored300,
                             ReverseScoredCustom)


def load_mock_answers_120() -> dict:
    with open(f"test/mock/answers-test-3.json") as f:
        data = json.load(f)
    return data


def load_mock_answers_300() -> dict:
    with open(f"test/mock/answers-test-4.json") as f:
        data = json.load(f)
    return data


def load_mock_answers_custom() -> dict:
    with open(f"test/mock/answers-test-6.json") as f:
        data = json.load(f)
    return data


class TestReverse(unittest.TestCase):
    def test_reverse_scored_120(self) -> None:
        with self.assertRaises(TypeError):
            ReverseScored120()

        with self.assertRaises(AssertionError) as e:
            ReverseScored120(answers=[])
        self.assertEqual(str(e.exception), "The (answers) field must be a dict!")

        with self.assertRaises(BaseException) as e:
            ReverseScored120(answers={})
        self.assertEqual(str(e.exception), "The key named (answers) was not found!")

        with self.assertRaises(BaseException) as e:
            ReverseScored120(answers={"answers": [{"id_select": 1}]})
        self.assertEqual(str(e.exception), "The key named (id_question) was not found!")

        with self.assertRaises(BaseException) as e:
            ReverseScored120(answers={"answers": [{"id_question": 1}]})
        self.assertEqual(str(e.exception), "The key named (id_select) was not found!")

        a = load_mock_answers_120()
        assert isinstance(a, dict), "a must be a dict"

        b = ReverseScored120(answers=load_mock_answers_120())
        assert isinstance(b, dict), "b must be a dict"
        assert isinstance(b.get("answers", []), list), "b must be a list"

        self.assertNotEqual(a, b)

    def test_reverse_scored_300(self) -> None:
        with self.assertRaises(TypeError):
            ReverseScored300()

        with self.assertRaises(AssertionError) as e:
            ReverseScored300(answers=[])
        self.assertEqual(str(e.exception), "The (answers) field must be a dict!")

        with self.assertRaises(BaseException) as e:
            ReverseScored300(answers={})
        self.assertEqual(str(e.exception), "The key named (answers) was not found!")

        with self.assertRaises(BaseException) as e:
            ReverseScored300(answers={"answers": [{"id_select": 1}]})
        self.assertEqual(str(e.exception), "The key named (id_question) was not found!")

        with self.assertRaises(BaseException) as e:
            ReverseScored300(answers={"answers": [{"id_question": 1}]})
        self.assertEqual(str(e.exception), "The key named (id_select) was not found!")

        a = load_mock_answers_300()
        assert isinstance(a, dict), "a must be a dict"

        b = ReverseScored300(answers=load_mock_answers_300())
        assert isinstance(b, dict), "b must be a dict"
        assert isinstance(b.get("answers", []), list), "b must be a list"

        self.assertNotEqual(a, b)

    def test_reverse_scored_custom(self) -> None:
        with self.assertRaises(TypeError):
            ReverseScoredCustom()

        with self.assertRaises(AssertionError) as e:
            ReverseScoredCustom(answers=[])
        self.assertEqual(str(e.exception), "The (answers) field must be a dict!")

        with self.assertRaises(BaseException) as e:
            ReverseScoredCustom(answers={})
        self.assertEqual(str(e.exception), "The key named (answers) was not found!")

        with self.assertRaises(BaseException) as e:
            ReverseScoredCustom(answers={"answers": [{"id_select": 1}]})
        self.assertEqual(str(e.exception), "The key named (id_question) was not found!")

        with self.assertRaises(BaseException) as e:
            ReverseScoredCustom(answers={"answers": [{"id_question": 1}]})
        self.assertEqual(str(e.exception), "The key named (id_select) was not found!")

        with self.assertRaises(BaseException) as e:
            ReverseScoredCustom(
                answers={"answers": [{"id_question": 1, "id_select": 1}]}
            )
        self.assertEqual(
            str(e.exception), "The key named (reverse_scored) was not found!"
        )

        a = load_mock_answers_custom()
        assert isinstance(a, dict), "a must be a dict"

        b = ReverseScoredCustom(answers=load_mock_answers_custom())
        assert isinstance(b, dict), "b must be a dict"
        assert isinstance(b.get("answers", []), list), "b must be a list"

        self.assertEqual(
            a.get("answers")[8].get("id_question"),
            b.get("answers")[8].get("id_question"),
        )
        self.assertEqual(
            a.get("answers")[8].get("reverse_scored"),
            b.get("answers")[8].get("reverse_scored"),
        )
        self.assertEqual(a.get("answers")[8].get("id_select"), 5)
        self.assertEqual(b.get("answers")[8].get("id_select"), 1)
        self.assertNotEqual(
            a.get("answers")[8].get("id_select"), b.get("answers")[8].get("id_select")
        )

        self.assertEqual(
            a.get("answers")[48].get("id_question"),
            b.get("answers")[48].get("id_question"),
        )
        self.assertEqual(
            a.get("answers")[48].get("reverse_scored"),
            b.get("answers")[48].get("reverse_scored"),
        )
        self.assertEqual(a.get("answers")[48].get("id_select"), 5)
        self.assertEqual(b.get("answers")[48].get("id_select"), 1)
        self.assertNotEqual(
            a.get("answers")[48].get("id_select"), b.get("answers")[48].get("id_select")
        )

        self.assertEqual(
            a.get("answers")[119].get("id_question"),
            b.get("answers")[119].get("id_question"),
        )
        self.assertEqual(
            a.get("answers")[119].get("reverse_scored"),
            b.get("answers")[119].get("reverse_scored"),
        )
        self.assertEqual(a.get("answers")[119].get("id_select"), 4)
        self.assertEqual(b.get("answers")[119].get("id_select"), 2)
        self.assertNotEqual(
            a.get("answers")[119].get("id_select"),
            b.get("answers")[119].get("id_select"),
        )

        self.assertNotEqual(a, b)



"""Reverses scores for certain questions from the IPIP-120 and IPIP-300."""

__author__ = "Ederson Corbari"
__email__ = "e@neural7.io"
__copyright__ = "Copyright Neural7 2022, Big 5 Personality Traits"
__credits__ = ["John A. Johnson", "Dhiru Kholia"]
__license__ = "MIT"
__version__ = "1.0.0"
__status__ = "production"

from ipipneo.utility import reverse_scored


class ReverseScoredCustom:
    """Reverse scored for Tests."""

    def __new__(self, answers: dict) -> dict or BaseException or AssertionError:
        """
        Apply reverse scoring to items with key (reverse_scored=1).

        Used to test reverse scored questions! Reverse-scored items were recoded
        (1=5, 2=4, 4=2, 5=1) at the time the respondent completed the inventory,
        so values for these items can simply be added without recoding whenscale
        scores are computed.

        Example position: [1, 2, 3, 4, 5] to [5, 4, 3, 2, 1].

        Args:
            - answers: Dictionary with the list of answers.
        """
        assert isinstance(answers, dict), "The (answers) field must be a dict!"

        if "answers" not in answers:
            raise BaseException("The key named (answers) was not found!")

        if not any("id_question" in x for x in answers.get("answers", [])):
            raise BaseException("The key named (id_question) was not found!")

        if not any("id_select" in x for x in answers.get("answers", [])):
            raise BaseException("The key named (id_select) was not found!")

        if not any("reverse_scored" in x for x in answers.get("answers", [])):
            raise BaseException("The key named (reverse_scored) was not found!")

        def is_reversed(x: dict) -> dict:
            x["id_select"] = (
                reverse_scored(select=x["id_select"])
                if x.get("reverse_scored") == 1
                else x["id_select"]
            )
            return x

        return {"answers": [is_reversed(x=x) for x in answers.get("answers", [])]}


class ReverseScored120:
    """Reverse scored for IPIP-120."""

    def __new__(self, answers: dict) -> dict or BaseException or AssertionError:
        """
        Apply reverse scoring on certain items (IPIP-120).

        Reverse-scored items were recoded (1=5, 2=4, 4=2, 5=1) at the time the respondent
        completed the inventory, so values for these items can simply be added without
        recoding whenscale scores are computed.

        Example position: [1, 2, 3, 4, 5] to [5, 4, 3, 2, 1].

        Args:
            - answers: Dictionary with the list of answers.
        """
        assert isinstance(answers, dict), "The (answers) field must be a dict!"

        if "answers" not in answers:
            raise BaseException("The key named (answers) was not found!")

        if not any("id_question" in x for x in answers.get("answers", [])):
            raise BaseException("The key named (id_question) was not found!")

        if not any("id_select" in x for x in answers.get("answers", [])):
            raise BaseException("The key named (id_select) was not found!")

        for i, x in enumerate(answers.get("answers", [])):
            if x.get("id_question", 0) == 9:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 19:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 24:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 30:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 39:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 40:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 48:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 49:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 51:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 53:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 54:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 60:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 62:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 67:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 68:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 69:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 70:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 73:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 74:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 75:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 78:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 79:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 80:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 81:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 83:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 84:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 85:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 88:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 89:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 90:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 92:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 94:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 96:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 97:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 98:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 99:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 100:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 101:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 102:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 103:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 104:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 105:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 106:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 107:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 108:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 109:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 110:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 111:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 113:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 114:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 115:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 116:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 118:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 119:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 120:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )

        return answers or {}


class ReverseScored300:
    """Reverse scored for IPIP-300."""

    def __new__(self, answers: dict) -> dict or BaseException or AssertionError:
        """
        Apply reverse scoring on certain items (IPIP-300).

        Reverse-scored items were recoded (1=5, 2=4, 4=2, 5=1) at the time the respondent
        completed the inventory, so values for these items can simply be added without
        recoding whenscale scores are computed.

        Example position: [1, 2, 3, 4, 5] to [5, 4, 3, 2, 1].

        Args:
            - answers: Dictionary with the list of answers.
        """
        assert isinstance(answers, dict), "The (answers) field must be a dict!"

        if "answers" not in answers:
            raise BaseException("The key named (answers) was not found!")

        if not any("id_question" in x for x in answers.get("answers", [])):
            raise BaseException("The key named (id_question) was not found!")

        if not any("id_select" in x for x in answers.get("answers", [])):
            raise BaseException("The key named (id_select) was not found!")

        for i, x in enumerate(answers.get("answers", [])):
            if x.get("id_question", 0) == 69:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 99:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 109:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 118:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 120:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 129:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 138:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 139:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 144:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 148:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 149:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 150:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 151:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 152:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 156:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 157:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 158:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 159:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 160:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 162:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 163:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 164:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 165:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 167:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 168:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 169:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 171:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 173:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 174:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 175:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 176:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 178:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 179:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 180:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 181:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 182:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 183:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 184:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 185:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 186:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 187:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 188:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 189:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 190:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 192:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 193:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 194:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 195:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 196:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 197:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 198:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 199:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 201:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 203:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 204:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 205:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 206:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 208:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 209:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 210:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 211:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 212:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 213:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 214:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 215:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 216:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 217:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 218:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 219:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 220:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 221:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 222:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 223:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 224:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 225:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 226:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 227:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 228:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 229:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 230:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 231:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 233:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 234:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 235:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 236:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 238:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 239:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 240:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 241:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 242:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 243:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 244:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 245:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 246:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 247:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 248:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 249:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 250:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 251:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 253:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 252:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 254:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 255:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 256:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 257:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 259:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 258:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 260:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 261:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 262:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 263:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 264:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 265:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 266:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 267:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 268:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 269:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 270:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 271:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 272:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 273:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 274:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 275:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 276:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 277:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 278:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 279:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 281:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 280:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 282:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 283:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 284:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 285:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 286:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 287:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 288:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 289:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 290:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 291:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 292:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 294:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 293:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 295:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 296:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 297:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 298:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 299:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )
            if x.get("id_question", 0) == 300:
                answers["answers"][i]["id_select"] = reverse_scored(
                    select=x.get("id_select", 0)
                )

        return answers or {}



"""The package settings."""

import codecs
import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = "1.5.0"
DESCRIPTION = "Big 5 IPIP-NEO Personality Traits"
LONG_DESCRIPTION = "Library with a self-administered questionnaire that assesses a person's personality according to the Big Five model, using the IPIP-NEO."

setup(
    name="five-factor-e",
    version=VERSION,
    author="Ederson Corbari, Neural7",
    author_email="<e@neural7.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(exclude=("test",)),
    python_requires=">=3.5",
    include_package_data=True,
    install_requires=[],
    extras_require={"quiz": ["plotext"]},
    entry_points={
        "console_scripts": [
            "ipipneo-quiz = ipipneo.quiz:main",
        ],
    },
    keywords=[
        "IPIP-NEO",
        "Big-5",
        "Big-Five",
        "Five-Factor",
        "Personality",
        "Assessment",
        "Psychometrics",
        "Personality-Insights",
        "People-Analytics",
        "Python",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
)



"""We use this for testing."""

__author__ = "Ederson Corbari"
__email__ = "e@neural7.io"
__copyright__ = "Copyright Neural7 2022, Big 5 Personality Traits"
__credits__ = ["John A. Johnson", "Dhiru Kholia"]
__license__ = "MIT"
__version__ = "1.2.0"
__status__ = "production"

import json
import random
import sys
import urllib.request
from itertools import chain, repeat

try:
    from ipipneo import IpipNeo
except ModuleNotFoundError:
    sys.path.insert(0, "../")
    from ipipneo.ipipneo import IpipNeo

URL_IPIP_QUESTIONS = (
    "https://raw.githubusercontent.com/neural7/five-factor-e/main/data/IPIP-NEO"
)


def load_ipip_questions(lang: int, question: int) -> dict:
    """
    Load the IPIP-NEO questions.

    Args:
        - lang: The language ID.
        - question: Inventory model 120 or 300.
    """
    target = URL_IPIP_QUESTIONS + "/" + str(question)
    if lang == 0:
        target = target + "/questions.json"
    if lang == 1:
        target = target + "/translation/questions-en-us.json"
    if lang == 2:
        target = target + "/translation/questions-pt-br.json"
    if lang == 3:
        target = target + "/translation/questions-es-es.json"
    try:
        return json.loads(urllib.request.urlopen(target).read())
    except BaseException:
        print(f"\nQuestion package not found in repository: {target}")
        sys.exit(0)


def get_questions(lang: int, question: int) -> list:
    """
    Filter only the question list.

    Args:
        - lang: The language ID.
        - question: Inventory model 120 or 300.
    """
    return [
        x
        for x in load_ipip_questions(lang=lang, question=question).get("questions", [])
    ]


def get_select(lang: int, question: int) -> list:
    """
    Filter only the select list.

    Args:
        - lang: The language ID.
        - question: Inventory model 120 or 300.
    """
    return [
        str(x.get("id", 0)) + ". " + str(x.get("text", None))
        for x in load_ipip_questions(lang=lang, question=question).get("select", [])
    ]


def question_translate() -> list:
    """Return the available question translations."""
    return ["0. IPIP-NEO Original", "1. EN-US", "2. PT-BR", "3. ES-ES"]


def quiz(inventory: int, sex: str, age: int, shuffle: str, lang: int) -> None:
    """
    Inventory / quiz of questions.

    Args:
        - inventory: Inventory model 120 or 300.
        - sex: The sex of the person.
        - age: The age of the person.
        - shuffle: Shuffle the questions.
        - lang: Then number of language.
    """
    questions, answers = (get_questions(lang=lang, question=inventory), [])
    print("\n *** Big Five IPIP-NEO Personality Test ***\n")

    if str(shuffle[0]).upper() == "Y":
        random.shuffle(questions)

    for i, q in enumerate(questions, start=1):
        print(f"\nQ.{i} {q.get('text')}\n")
        print(*get_select(lang=lang, question=inventory), sep="\n")

        replies = map(
            input,
            chain(
                ["\nOption: "],
                repeat("Only numbers from 1 to 5 are accepted! Try again: "),
            ),
        )
        option = int(next(filter({"1", "2", "3", "4", "5"}.__contains__, replies)))
        answers.append({"id_question": q.get("id"), "id_select": option})

    result = IpipNeo(question=inventory).compute(
        sex=sex, age=age, answers={"answers": answers}, compare=True
    )

    object = json.dumps(result, indent=4)
    fname = f"result-{str(inventory)}-{result.get('id', 'id')}.json"

    with open(fname, "w") as f:
        f.write(object)

    plot_results(result=result)

    print(f"\nThe result was saved in the file: {fname}")


def plot_results(result: dict) -> None:
    """
    Show the Big-Five result plots.

    Args:
        - result: Result of the person's big-five.
    """
    try:
        import plotext as plt

        big5 = result.get("person").get("result").get("personalities")

        print(
            "\nInventory:",
            str(result.get("model"))
            + "-"
            + str(result.get("question"))
            + " v"
            + str(result.get("version")),
        )
        print("Case:", result.get("id"))
        print("Gender:", result.get("person").get("sex"))
        print("Age:", result.get("person").get("age"), "\n")

        E = [
            "EXTRAVERSION",
            "Friendliness",
            "Gregariousness",
            "Assertiveness",
            "Activity Level",
            "Excitement-Seeking",
            "Cheerfulness",
        ]

        plt.simple_bar(
            E,
            [
                int(big5[2].get("Extraversion").get("E")),
                int(big5[2].get("Extraversion").get("traits")[0].get("Friendliness")),
                int(big5[2].get("Extraversion").get("traits")[1].get("Gregariousness")),
                int(big5[2].get("Extraversion").get("traits")[2].get("Assertiveness")),
                int(big5[2].get("Extraversion").get("traits")[3].get("Activity-Level")),
                int(
                    big5[2]
                    .get("Extraversion")
                    .get("traits")[4]
                    .get("Excitement-Seeking")
                ),
                int(big5[2].get("Extraversion").get("traits")[5].get("Cheerfulness")),
            ],
            width=100,
            title="Big-Five | Extraversion",
            color="orange",
        )
        plt.show()

        print("\n")

        A = [
            "AGREEABLENESS",
            "Trust",
            "Morality",
            "Altruism",
            "Cooperation",
            "Modesty",
            "Sympathy",
        ]

        plt.simple_bar(
            A,
            [
                int(big5[3].get("Agreeableness").get("A")),
                int(big5[3].get("Agreeableness").get("traits")[0].get("Trust")),
                int(big5[3].get("Agreeableness").get("traits")[1].get("Morality")),
                int(big5[3].get("Agreeableness").get("traits")[2].get("Altruism")),
                int(big5[3].get("Agreeableness").get("traits")[3].get("Cooperation")),
                int(big5[3].get("Agreeableness").get("traits")[4].get("Modesty")),
                int(big5[3].get("Agreeableness").get("traits")[5].get("Sympathy")),
            ],
            width=100,
            title="Big-Five | Agreeableness",
            color="green",
        )
        plt.show()

        print("\n")

        C = [
            "CONSCIENTIOUSNESS",
            "Self-Efficacy",
            "Orderliness",
            "Dutifulness",
            "Achievement-Striving",
            "Self-Discipline",
            "Cautiousness",
        ]

        plt.simple_bar(
            C,
            [
                int(big5[1].get("Conscientiousness").get("C")),
                int(
                    big5[1]
                    .get("Conscientiousness")
                    .get("traits")[0]
                    .get("Self-Efficacy")
                ),
                int(
                    big5[1].get("Conscientiousness").get("traits")[1].get("Orderliness")
                ),
                int(
                    big5[1].get("Conscientiousness").get("traits")[2].get("Dutifulness")
                ),
                int(
                    big5[1]
                    .get("Conscientiousness")
                    .get("traits")[3]
                    .get("Achievement-Striving")
                ),
                int(
                    big5[1]
                    .get("Conscientiousness")
                    .get("traits")[4]
                    .get("Self-Discipline")
                ),
                int(
                    big5[1]
                    .get("Conscientiousness")
                    .get("traits")[5]
                    .get("Cautiousness")
                ),
            ],
            width=100,
            title="Big-Five | Consciousness",
            color="magenta",
        )
        plt.show()

        print("\n")

        N = [
            "NEUROTICISM",
            "Anxiety",
            "Anger",
            "Depression",
            "Self-Consciousness",
            "Immoderation",
            "Vulnerability",
        ]

        plt.simple_bar(
            N,
            [
                int(big5[4].get("Neuroticism").get("N")),
                int(big5[4].get("Neuroticism").get("traits")[0].get("Anxiety")),
                int(big5[4].get("Neuroticism").get("traits")[1].get("Anger")),
                int(big5[4].get("Neuroticism").get("traits")[2].get("Depression")),
                int(
                    big5[4]
                    .get("Neuroticism")
                    .get("traits")[3]
                    .get("Self-Consciousness")
                ),
                int(big5[4].get("Neuroticism").get("traits")[4].get("Immoderation")),
                int(big5[4].get("Neuroticism").get("traits")[5].get("Vulnerability")),
            ],
            width=100,
            title="Big-Five | Neuroticism",
            color="red",
        )
        plt.show()

        print("\n")

        O = [
            "OPENNESS",
            "Imagination",
            "Artistic Interests",
            "Emotionality",
            "Adventurousness",
            "Intellect",
            "Liberalism",
        ]

        plt.simple_bar(
            O,
            [
                int(big5[0].get("Openness").get("O")),
                int(big5[0].get("Openness").get("traits")[0].get("Imagination")),
                int(big5[0].get("Openness").get("traits")[1].get("Artistic-Interests")),
                int(big5[0].get("Openness").get("traits")[2].get("Emotionality")),
                int(big5[0].get("Openness").get("traits")[3].get("Adventurousness")),
                int(big5[0].get("Openness").get("traits")[4].get("Intellect")),
                int(big5[0].get("Openness").get("traits")[5].get("Liberalism")),
            ],
            width=100,
            title="Big-Five | Openness",
            color="blue",
        )
        plt.show()

    except ModuleNotFoundError:
        print("\n*** Unable to plot graphs! Install the package (plotext). ***\n")


def main() -> None:
    """Initialize the test."""
    print("\n *** Welcome to Big Five IPIP-NEO Personality Test ***")

    yes_or_no = {"Y", "y", "yes", "Yes", "YES", "N", "n", "no", "No", "NO"}

    replies = map(
        input,
        chain(
            ["\n> Do you wish to continue? Yes / No: "],
            repeat("Please, only Yes or No are valid! Try again: "),
        ),
    )
    go = str(next(filter(yes_or_no.__contains__, replies)))

    if go.upper() == "NO" or go.upper() == "N":
        print("Bye!")
        sys.exit(0)

    replies = map(
        input,
        chain(
            ["\n> What inventory model will you make? 120 / 300: "],
            repeat("Please, only 120 or 300 are valid! Try again: "),
        ),
    )
    inventory = int(next(filter({"120", "300"}.__contains__, replies)))

    print("\n====================================================================")
    if inventory == 120:
        print(
            "The following test contains 120 questions which is estimated to take you about 15 minutes to complete!"
        )
    elif inventory == 300:
        print(
            "The following test contains 300 questions which is estimated to take you about 35 minutes to complete!"
        )

    replies = map(
        input,
        chain(
            ["\n> What is your sex Male or Female? M / F: "],
            repeat("Please, only M or F are valid! Try again: "),
        ),
    )
    sex = str(next(filter({"F", "f", "M", "m"}.__contains__, replies)))

    replies = map(
        input,
        chain(
            ["\n> What is your age? "],
            repeat("Please, only numbers between 18 and 100 are valid! Try again: "),
        ),
    )
    age = int(next(filter(set(map(str, range(18, 101))).__contains__, replies)))

    replies = map(
        input,
        chain(
            ["\n> Do you want to shuffle the questions? Yes / No: "],
            repeat("Please, only Yes or No are valid! Try again: "),
        ),
    )
    shuffle = str(next(filter(yes_or_no.__contains__, replies)))

    print("\n====================================================================")
    print(*question_translate(), sep="\n")

    replies = map(
        input,
        chain(
            ["\n> Choose the language of the questions above: "],
            repeat(
                "Please, only the numbers that are on the list are valid! Try again: "
            ),
        ),
    )
    lang = int(next(filter(set(map(str, range(0, 3))).__contains__, replies)))

    quiz(
        inventory=int(inventory),
        sex=str(sex).upper(),
        age=int(age),
        shuffle=str(shuffle).upper(),
        lang=int(lang),
    )


if __name__ == "__main__":
    main()
